#ifndef _ANGLE_H_
#define _ANGLE_H_

//! \file   ~/sw/modules/angle/src/float/angle.h
//! \brief  Contains the public interface to the angle generator (ANGLE)
//!         module routines
//!
//! (C) Copyright 2014, Texas Instruments, Inc.


// **************************************************************************
// the includes
#include <stdint.h>

// drivers 


// modules
#include "sw/modules/types/src/types.h"
#include "sw/modules/est/src/float/est.h"
#include "sw/modules/math/src/float/math.h"


// solutions


//!
//! \defgroup ANGLE

//!
//! \ingroup ANGLE
//@{


#ifdef __cplusplus
extern "C" {
#endif


// **************************************************************************
// the defines


// **************************************************************************
// the typedefs


//! \brief Defines the angle generator (ANGLE) object
//!
typedef struct _ANGLE_Obj_
{
  int_least32_t  timeStamp;             //!< the time stamp for the estimated angle and speed values
  float_t        angleEst_rad;          //!< the estimated angle value, rad
  float_t        speedEst_rps;          //!< the estimated angular speed value, rad/sec
  float_t        accelEst_rps2;         //!< the estimated angular acceleration value, rad/sec^2

  float_t        angle_rad;             //!< the predicted angle value, rad
  float_t        angleWithDelay_rad;    //!< the predicted angle value considering system delay, rad
  float_t        speed_rps;             //!< the predicted angular speed value, rad/sec

  float_t        angleDelayed_sf_sec;   //!< the scale factor for computing angle value considering system delay, sec
  float_t        timeStampPeriod_sec;   //!< the time stamp period, sec
} ANGLE_Obj;


//! \brief Defines the ANGLE handle
//!
typedef struct _ANGLE_Obj_  *ANGLE_Handle;


// **************************************************************************
// the globals


// **************************************************************************
// the function prototypes

//! \brief     Gets the estimated acceleration value 
//! \param[in] handle  The estimator (EST) handle
//! \return    The estimated acceleration value, rad/sec^2
static inline float_t ANGLE_getAccelEst_rps2(ANGLE_Handle handle)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  return(obj->accelEst_rps2);
} // end of ANGLE_getAccelEst_rps2() function


//! \brief     Gets the predicted angle value
//! \param[in] handle  The angle generator (ANGLE) handle
//! \return    The predicted angle value, rad
static inline float_t ANGLE_getAngle_rad(ANGLE_Handle handle)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  return(obj->angle_rad);
} // end of ANGLE_getAngle_rad() function


//! \brief     Gets the estimated angle value
//! \param[in] handle  The angle generator (ANGLE) handle
//! \return    The estimated angle value, rad
static inline float_t ANGLE_getAngleEst_rad(ANGLE_Handle handle)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  return(obj->angleEst_rad);
} // end of ANGLE_getAngleEst_rad() function


//! \brief     Gets the updated angle value considering system delay
//! \param[in] handle  The angle generator (ANGLE) handle
//! \return    The updated angle value considering delay, rad
static inline float_t ANGLE_getAngleWithDelay_rad(ANGLE_Handle handle)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  return(obj->angleWithDelay_rad);
} // end of ANGLE_getAngleWithDelay_rad() function


//! \brief     Gets the scale factor for computing the delayed angle
//! \param[in] handle  The angle generator (ANGLE) handle
//! \return    The scale factor, sec
static inline float_t ANGLE_getAngleDelayed_sf_sec(ANGLE_Handle handle)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  return(obj->angleDelayed_sf_sec);
} // end of ANGLE_getAngleDelayed_sf_sec() function


//! \brief     Gets the predicted speed value 
//! \param[in] handle  The estimator (EST) handle
//! \return    The predicted speed value, rad/sec
static inline float_t ANGLE_getSpeed_rps(ANGLE_Handle handle)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  return(obj->speed_rps);
} // end of ANGLE_getSpeed_rps() function


//! \brief     Gets the estimated speed value 
//! \param[in] handle  The estimator (EST) handle
//! \return    The estimated speed value, rad/sec
static inline float_t ANGLE_getSpeedEst_rps(ANGLE_Handle handle)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  return(obj->speedEst_rps);
} // end of ANGLE_getSpeedEst_rps() function


//! \brief     Gets the time stamp value for the estimated data
//! \param[in] handle  The angle generator (ANGLE) handle
//! \return    The time stamp value
static inline int_least32_t ANGLE_getTimeStamp(ANGLE_Handle handle)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  return(obj->timeStamp);
} // end of ANGLE_getTimeStamp() function


//! \brief     Gets the time stamp period value
//! \param[in] handle  The estimator (EST) handle
//! \return    The time stamp period, sec
static inline float_t ANGLE_getTimeStampPeriod_sec(ANGLE_Handle handle)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  return(obj->timeStampPeriod_sec);
} // end of ANGLE_getTimeStampPeriod_sec() function


//! \brief     Initializes the angle generator (ANGLE) module
//! \param[in] pMemory   A pointer to the memory for the object
//! \param[in] numBytes  The number of bytes allocated for the object, bytes
//! \return    The angle generator (ANGLE) object handle
extern ANGLE_Handle     ANGLE_init(void *pMemory,const size_t numBytes);
extern ANGLE_Handle cla_ANGLE_init(void *pMemory,const size_t numBytes);


//! \brief     Updates the angle value in the angle generator
//! \param[in] handle     The angle generator (ANGLE) handle
//! \param[in] timeStamp  The time stamp for execution
extern void     ANGLE_run(ANGLE_Handle handle,const int_least32_t timeStamp);
extern void cla_ANGLE_run(ANGLE_Handle handle,const int_least32_t timeStamp);


//! \brief     Sets the estimated acceleration value
//! \param[in] handle      The angle generator (ANGLE) handle
//! \param[in] accel_rps2  The estimated acceleration value, rad/sec2
static inline void ANGLE_setAccelEst_rps2(ANGLE_Handle handle,const float_t accel_rps2)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  obj->accelEst_rps2 = accel_rps2;

  return;
} // end of ANGLE_setAccelEst_rps() function


//! \brief     Sets the predicted angle value
//! \param[in] handle     The angle generator (ANGLE) handle
//! \param[in] angle_rad  The angle value, rad
static inline void ANGLE_setAngle_rad(ANGLE_Handle handle,const float_t angle_rad)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  obj->angle_rad = angle_rad;

  return;
} // end of ANGLE_setAngle_rad() function


//! \brief     Sets the estimated angle value
//! \param[in] handle     The angle generator (ANGLE) handle
//! \param[in] angle_rad  The estimated angle value, rad
static inline void ANGLE_setAngleEst_rad(ANGLE_Handle handle,const float_t angle_rad)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  obj->angleEst_rad = angle_rad;

  return;
} // end of ANGLE_setAngleEst_rad() function


//! \brief     Sets the angle value considering system delay
//! \param[in] handle     The angle generator (ANGLE) handle
//! \param[in] angle_rad  The angle value with system delay, rad
static inline void ANGLE_setAngleWithDelay_rad(ANGLE_Handle handle,const float_t angle_rad)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  obj->angleWithDelay_rad = angle_rad;

  return;
} // end of ANGLE_setAngleWithDelay_rad() function


//! \brief     Sets the scale factor for computing the delayed angle
//! \param[in] handle               The angle generator (ANGLE) handle
//! \param[in] angleDelayed_sf_sec  The scale factor
static inline void ANGLE_setAngleDelayed_sf_sec(ANGLE_Handle handle,const float_t angleDelayed_sf_sec)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  obj->angleDelayed_sf_sec = angleDelayed_sf_sec;

  return;
} // end of ANGLE_setAngleDelayed_sf_sec() function


//! \brief     Sets the parameters
//! \param[in] handle               The angle generator (ANGLE) handle
//! \param[in] timeStampPeriod_sec  The time difference between two consecutive time stamp values
//! \param[in] angleDelayed_sf_sec  The scale factor for computing the delayed angle
extern void     ANGLE_setParams(ANGLE_Handle handle,
                                const float_t timeStampPeriod_sec,
                                const float_t angleDelayed_sf_sec);
extern void cla_ANGLE_setParams(ANGLE_Handle handle,
                                const float_t timeStampPeriod_sec,
                                const float_t angleDelayed_sf_sec);


//! \brief     Sets the predicted speed value
//! \param[in] handle     The angle generator (ANGLE) handle
//! \param[in] speed_rps  The speed value, rad/sec
static inline void ANGLE_setSpeed_rps(ANGLE_Handle handle,const float_t speed_rps)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  obj->speed_rps = speed_rps;

  return;
} // end of ANGLE_setSpeed_rps() function


//! \brief     Sets the estimated speed value
//! \param[in] handle     The angle generator (ANGLE) handle
//! \param[in] speed_rps  The speed value, rad/sec
static inline void ANGLE_setSpeedEst_rps(ANGLE_Handle handle,const float_t speed_rps)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  obj->speedEst_rps = speed_rps;

  return;
} // end of ANGLE_setSpeedEst_rps() function


//! \brief     Sets the time stamp value for the estimated data
//! \param[in] handle     The angle generator (ANGLE) handle
//! \param[in] timeStamp  The time stamp value
static inline void ANGLE_setTimeStamp(ANGLE_Handle handle,const int_least32_t timeStamp)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  obj->timeStamp = timeStamp;

  return;
} // end of ANGLE_setTimeStamp() function


//! \brief     Sets the time stamp period value
//! \param[in] handle               The angle generator (ANGLE) handle
//! \param[in] timeStampPeriod_sec  The time stamp period, sec
static inline void ANGLE_setTimeStampPeriod_sec(ANGLE_Handle handle,const float_t timeStampPeriod_sec)
{
  ANGLE_Obj *obj = (ANGLE_Obj *)handle;

  obj->timeStampPeriod_sec = timeStampPeriod_sec;

  return;
} // end of ANGLE_setTimeStampPeriod_sec() function


//! \brief     Updates the angle generator with new data
//! \param[in] handle      The angle generator (ANGLE) handle
//! \param[in] timeStamp   The time stamp for the angle and speed estimates
//! \param[in] angle_rad   The estimated angle value, rad
//! \param[in] speed_rps   The estimated speed value, rad/sec
//! \param[in] accel_rps2  The estimated acceleration value, rad/sec^2
extern void     ANGLE_setup(ANGLE_Handle handle,
                            const int_least32_t timeStamp,
                            const float_t angle_rad,
                            const float_t speed_rps,
                            const float_t accel_rps2);
extern void cla_ANGLE_setup(ANGLE_Handle handle,
                            const int_least32_t timeStamp,
                            const float_t angle_rad,
                            const float_t speed_rps,
                            const float_t accel_rps2);


#ifdef __cplusplus
}
#endif // extern "C"

//@}  // ingroup

#endif // end of _ANGLE_H_ definition


