//! \file   modules/offset/test/f28x/f2806x/test/src/hal.c
//! \brief  Contains the various functions related to the HAL object
//!
//! (C) Copyright 2011, Texas Instruments, Inc.


// **************************************************************************
// the includes

// drivers

// modules

// platforms
#include "sw/modules/offset/boards/docking_station_rev3/f28x/f2806x/test/src/hal.h"


// **************************************************************************
// the defines


// **************************************************************************
// the globals

HAL_Obj hal;


// **************************************************************************
// the functions

void HAL_cal(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;


  // enable the ADC clock
  CLK_enableAdcClock(hal->clkHandle);


  // Run the Device_cal() function
  // This function copies the ADC and oscillator calibration values from TI reserved
  // OTP into the appropriate trim registers
  // This boot ROM automatically calls this function to calibrate the interal 
  // oscillators and ADC with device specific calibration data.
  // If the boot ROM is bypassed by Code Composer Studio during the development process,
  // then the calibration must be initialized by the application
  (*Device_cal)();


  // disable the ADC clock
  CLK_disableAdcClock(hal->clkHandle);

  return;
} // end of HAL_cal() function


void HAL_disableGlobalInts(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;


  CPU_disableGlobalInts(hal->cpuHandle);

  return;
} // end of HAL_disableGlobalInts() function


void HAL_enableDebugInt(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;


  CPU_enableDebugInt(hal->cpuHandle);

  return;
} // end of HAL_enableDebugInt() function


void HAL_enableGlobalInts(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;


  CPU_enableGlobalInts(hal->cpuHandle);

  return;
} // end of HAL_enableGlobalInts() function


void HAL_enableTimerInt(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;


  PIE_enableTimer0Int(hal->pieHandle);


  // enable the interrupt
  TIMER_enableInt(hal->timerHandle);

  // enable the cpu interrupt for TINT0_INT
  CPU_enableInt(hal->cpuHandle,CPU_IntNumber_1);

  return;
} // end of HAL_enablePwmInt() function


HAL_Handle HAL_init(void *pMemory,const size_t numBytes)
{
  HAL_Handle halHandle;
  HAL_Obj *hal;


  if(numBytes < sizeof(HAL_Obj))
    return((HAL_Handle)NULL);


  // assign the handle
  halHandle = (HAL_Handle)pMemory;


  // assign the object
  hal = (HAL_Obj *)halHandle;


  // initialize the clock handle
  hal->clkHandle = CLK_init((void *)CLK_BASE_ADDR,sizeof(CLK_Obj));


  // initialize the CPU handle
  hal->cpuHandle = CPU_init(&cpu,sizeof(cpu));


  // initialize the FLASH handle
  hal->flashHandle = FLASH_init((void *)FLASH_BASE_ADDR,sizeof(FLASH_Obj));


  // initialize the GPIO handle
  hal->gpioHandle = GPIO_init((void *)GPIO_BASE_ADDR,sizeof(GPIO_Obj));


  // initialize the oscillator handle
  hal->oscHandle = OSC_init((void *)OSC_BASE_ADDR,sizeof(OSC_Obj));


  // initialize the PIE handle
  hal->pieHandle = PIE_init((void *)PIE_BASE_ADDR,sizeof(PIE_Obj));


  // initialize the PLL handle
  hal->pllHandle = PLL_init((void *)PLL_BASE_ADDR,sizeof(PLL_Obj));


  // initialize power handle
  hal->pwrHandle = PWR_init((void *)PWR_BASE_ADDR,sizeof(PWR_Obj));


  // initialize timer drivers
  hal->timerHandle = TIMER_init((void *)TIMER0_BASE_ADDR,sizeof(TIMER_Obj));


  // initialize the watchdog driver
  hal->wdogHandle = WDOG_init((void *)WDOG_BASE_ADDR,sizeof(WDOG_Obj));


  return(halHandle);
} // end of HAL_init() function


void HAL_setup(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;


  // disable the watchdog 
  WDOG_disable(hal->wdogHandle);


  // disable global interrupts
  CPU_disableGlobalInts(hal->cpuHandle);


  // disable cpu interrupts
  CPU_disableInts(hal->cpuHandle);


  // clear cpu interrupt flags
  CPU_clearIntFlags(hal->cpuHandle);


  // setup the clocks
  HAL_setupClks(halHandle);


  // Setup the PLL
  HAL_setupPll(halHandle,PLL_ClkFreq_80_MHz);


  // setup the PIE
  HAL_setupPie(halHandle);


  // run the device calibration
  HAL_cal(halHandle);


  // setup the peripheral clocks
  HAL_setupPeripheralClks(halHandle);


  // setup the GPIOs
  HAL_setupGpios(halHandle);


  // setup the flash
  HAL_setupFlash(halHandle);

  return;
} // end of HAL_setup() function


void HAL_setupClks(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;


  // enable internal oscillator 1
  CLK_enableOsc1(hal->clkHandle);

  // set the oscillator source
  CLK_setOscSrc(hal->clkHandle,CLK_OscSrc_Internal);

  // disable the external clock in
  CLK_disableClkIn(hal->clkHandle);

  // disable the crystal oscillator
  CLK_disableCrystalOsc(hal->clkHandle);

  // disable oscillator 2
  CLK_disableOsc2(hal->clkHandle);

  // set the low speed clock prescaler
  CLK_setLowSpdPreScaler(hal->clkHandle,CLK_LowSpdPreScaler_SysClkOut_by_4);

  // set the clock out prescaler
  CLK_setClkOutPreScaler(hal->clkHandle,CLK_ClkOutPreScaler_SysClkOut_by_1);

  return;
} // end of HAL_setupClks() function


void HAL_setupFlash(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;


  FLASH_enablePipelineMode(hal->flashHandle);

  FLASH_setNumPagedReadWaitStates(hal->flashHandle,FLASH_NumPagedWaitStates_3);

  FLASH_setNumRandomReadWaitStates(hal->flashHandle,FLASH_NumRandomWaitStates_3);

  FLASH_setOtpWaitStates(hal->flashHandle,FLASH_NumOtpWaitStates_5);

  FLASH_setStandbyWaitCount(hal->flashHandle,FLASH_STANDBY_WAIT_COUNT_DEFAULT);

  FLASH_setActiveWaitCount(hal->flashHandle,FLASH_ACTIVE_WAIT_COUNT_DEFAULT);

  return;
} // HAL_setupFlash() function


void HAL_setupGpios(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;

  // LED1
  GPIO_setMode(hal->gpioHandle,GPIO_Number_12,GPIO_12_Mode_GeneralPurpose);
  GPIO_setHigh(hal->gpioHandle,GPIO_Number_12);
  GPIO_setDirection(hal->gpioHandle,GPIO_Number_12,GPIO_Direction_Output);

  // LED2
  GPIO_setMode(hal->gpioHandle,GPIO_Number_15,GPIO_15_Mode_GeneralPurpose);
  GPIO_setHigh(hal->gpioHandle,GPIO_Number_15);
  GPIO_setDirection(hal->gpioHandle,GPIO_Number_15,GPIO_Direction_Output);

  return;
}  // end of HAL_setupGpios() function


void HAL_setupPie(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;


  PIE_disable(hal->pieHandle);

  PIE_disableAllInts(hal->pieHandle);

  PIE_clearAllInts(hal->pieHandle);

  PIE_clearAllFlags(hal->pieHandle);

  PIE_setDefaultIntVectorTable(hal->pieHandle);

  PIE_enable(hal->pieHandle);

  return;
} // end of HAL_setupPie() function


void HAL_setupPeripheralClks(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;


  CLK_disableAdcClock(hal->clkHandle);

  CLK_disableCompClock(hal->clkHandle,CLK_CompNumber_1);
  CLK_disableCompClock(hal->clkHandle,CLK_CompNumber_2);
  CLK_disableCompClock(hal->clkHandle,CLK_CompNumber_3);

  CLK_disableEcap1Clock(hal->clkHandle);

  CLK_disableEcanaClock(hal->clkHandle);

  CLK_disableEqep1Clock(hal->clkHandle);

  CLK_disablePwmClock(hal->clkHandle,PWM_Number_1);
  CLK_disablePwmClock(hal->clkHandle,PWM_Number_2);
  CLK_disablePwmClock(hal->clkHandle,PWM_Number_3);
  CLK_disablePwmClock(hal->clkHandle,PWM_Number_4);
  CLK_disablePwmClock(hal->clkHandle,PWM_Number_5);
  CLK_disablePwmClock(hal->clkHandle,PWM_Number_6);
  CLK_disablePwmClock(hal->clkHandle,PWM_Number_7);

  CLK_disableHrPwmClock(hal->clkHandle);

  CLK_disableI2cClock(hal->clkHandle);

  CLK_disableLinAClock(hal->clkHandle);

  CLK_disableClaClock(hal->clkHandle);

  CLK_disableSciaClock(hal->clkHandle);

  CLK_disableSpiaClock(hal->clkHandle);
  CLK_disableSpibClock(hal->clkHandle);
  
  CLK_enableTbClockSync(hal->clkHandle);

  return;
} // end of HAL_setupPeripheralClks() function


void HAL_setupPll(HAL_Handle halHandle,const PLL_ClkFreq_e clkFreq)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;


  // make sure PLL is not running in limp mode
  if(PLL_getClkStatus(hal->pllHandle) != PLL_ClkStatus_Normal)
    {
      // reset the clock detect
      PLL_resetClkDetect(hal->pllHandle);

      // ???????
      asm("        ESTOP0");
    }


  // Divide Select must be ClkIn/4 before the clock rate can be changed
  if(PLL_getDivideSelect(hal->pllHandle) != PLL_DivideSelect_ClkIn_by_4)
    {
      PLL_setDivideSelect(hal->pllHandle,PLL_DivideSelect_ClkIn_by_4);
    }


  if(PLL_getClkFreq(hal->pllHandle) != clkFreq)
    {
      // disable the clock detect
      PLL_disableClkDetect(hal->pllHandle);

      // set the clock rate
      PLL_setClkFreq(hal->pllHandle,clkFreq);
    }


  // wait until locked
  while(PLL_getLockStatus(hal->pllHandle) != PLL_LockStatus_Done) {}


  // enable the clock detect
  PLL_enableClkDetect(hal->pllHandle);


  // set divide select to ClkIn/2 to get desired clock rate
  // NOTE: clock must be locked before setting this register
  PLL_setDivideSelect(hal->pllHandle,PLL_DivideSelect_ClkIn_by_2);

  return;
} // end of HAL_setupPll() function


void HAL_setupTimer(HAL_Handle halHandle, uint32_t period)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;

  TIMER_setPeriod(hal->timerHandle, period);
  TIMER_start(hal->timerHandle);

  return;
} // end of HAL_setupTimer() function

// end of file
