#ifndef _HAL_H_
#define _HAL_H_

//! \file   modules/offset/test/f28x/f2806x/test/src/hal.h
//! \brief  Contains public interface to various functions related
//!         to the HAL object
//!
//! (C) Copyright 2011, Texas Instruments, Inc.


// **************************************************************************
// the includes


// drivers
#include "sw/drivers/clk/src/32b/f28x/f2806x/clk.h"
#include "sw/drivers/cpu/src/32b/f28x/f2806x/cpu.h"
#include "sw/drivers/flash/src/32b/f28x/f2806x/flash.h"
#include "sw/drivers/gpio/src/32b/f28x/f2806x/gpio.h"
#include "sw/drivers/osc/src/32b/f28x/f2806x/osc.h"
#include "sw/drivers/pie/src/32b/f28x/f2806x/pie.h"
#include "sw/drivers/pll/src/32b/f28x/f2806x/pll.h"
#include "sw/drivers/pwr/src/32b/f28x/f2806x/pwr.h"
#include "sw/drivers/timer/src/32b/f28x/f2806x/timer.h"
#include "sw/drivers/wdog/src/32b/f28x/f2806x/wdog.h"


// modules


// platforms


//!
//!
//! \defgroup HAL HAL
//!
//@{


#ifdef __cplusplus
extern "C" {
#endif


// **************************************************************************
// the defines

#define Device_cal (void   (*)(void))0x3D7C80


#define HAL_turnLedOff            HAL_setGpioLow
#define HAL_turnLedOn             HAL_setGpioHigh


// **************************************************************************
// the typedefs


typedef enum
{
  HAL_Gpio_LEDstatus=GPIO_Number_22,  //!< GPIO pin number for DRV8301 Status LED
  HAL_Gpio_LED2=GPIO_Number_31,  //!< GPIO pin number for ControlCARD LED 2
  HAL_Gpio_LED3=GPIO_Number_34   //!< GPIO pin number for ControlCARD LED 3
} HAL_LedNumber_e;


//! \brief Defines the hardware abstraction layer (HAL) data
//!
typedef struct _HAL_Obj_
{
  CLK_Handle    clkHandle;        //!< the clock handle
 
  CPU_Handle    cpuHandle;      //!< the CPU handle

  FLASH_Handle  flashHandle;      //!< the flash handle

  GPIO_Handle   gpioHandle;       //!< the GPIO handle

  OSC_Handle    oscHandle;        //!< the oscillator handle

  PIE_Handle    pieHandle;        //<! the PIE handle

  PLL_Handle    pllHandle;        //!< the PLL handle

  PWR_Handle    pwrHandle;        //<! the power handle

  TIMER_Handle  timerHandle;   	  //<! the timer handles

  WDOG_Handle   wdogHandle;       //!< the watchdog handle

} HAL_Obj;


//! \brief Defines the HAL handle
//!
typedef struct _HAL_Obj_ *HAL_Handle;


// **************************************************************************
// the globals

//! \brief Defines the HAL object
//!
extern HAL_Obj hal;


extern interrupt void mainISR(void);


// **************************************************************************
// the function prototypes


void HAL_cal(HAL_Handle halHandle);


//! \brief     Disables global interrupts
//! \param[in] halHandle  The hardware abstraction layer handle
void HAL_disableGlobalInts(HAL_Handle halHandle);


//! \brief     Enables the debug interrupt
//! \param[in] halHandle  The hardware abstraction layer handle
void HAL_enableDebugInt(HAL_Handle halHandle);


//! \brief     Enables global interrupts
//! \param[in] halHandle  The hardware abstraction layer handle
void HAL_enableGlobalInts(HAL_Handle halHandle);


void HAL_enableTimerInt(HAL_Handle halHandle);


//! \brief     Initializes the hardware abstraction layer (HAL) object
//! \param[in] pMemory     A pointer to the memory for the hardware abstraction layer object
//! \param[in] numBytes    The number of bytes allocated for the hardware abstraction layer object, bytes
//! \return    The hardware abstraction layer (HAL) object handle
HAL_Handle HAL_init(void *pMemory,const size_t numBytes);


inline void HAL_acqTimer0Int(HAL_Handle halHandle)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;

  // Acknowledge interrupt from PIE group 1
  PIE_clearInt(hal->pieHandle,PIE_GroupNumber_1);

  return;
} // end of HAL_acqAdcInt() function


//! \brief     Initializes the interrupt vector table
//! \param[in] halHandle  The hardware abstraction layer handle
inline void HAL_initIntVectorTable(HAL_Handle halHandle)
 {
  HAL_Obj *hal = (HAL_Obj *)halHandle;
  PIE_Obj *pie = (PIE_Obj *)hal->pieHandle;


  ENABLE_PROTECTED_REGISTER_WRITE_MODE;

  pie->TINT0 = &mainISR;

  DISABLE_PROTECTED_REGISTER_WRITE_MODE;

  return;
 } // end of HAL_initIntVectorTable() function


inline void HAL_setGpioHigh(HAL_Handle halHandle,const GPIO_Number_e gpioNumber)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;
  

  // set GPIO high
  GPIO_setHigh(hal->gpioHandle,gpioNumber);

  return;
} // end of HAL_setGpioHigh() function


inline void HAL_setGpioLow(HAL_Handle halHandle,const GPIO_Number_e gpioNumber)
{
  HAL_Obj *hal = (HAL_Obj *)halHandle;
  

  // set GPIO low
  GPIO_setLow(hal->gpioHandle,gpioNumber);

  return;
} // end of HAL_setGpioLow() function


void HAL_setup(HAL_Handle halHandle);


void HAL_setupClks(HAL_Handle halHandle);


void HAL_setupFlash(HAL_Handle halHandle);


void HAL_setupGpios(HAL_Handle halHandle);


void HAL_setupPeripheralClks(HAL_Handle halHandle);


void HAL_setupPie(HAL_Handle halHandle);


void HAL_setupPll(HAL_Handle halHandle,const PLL_ClkFreq_e clkFreq);


void HAL_setupTimer(HAL_Handle halHandle, uint32_t period);


#ifdef __cplusplus
}
#endif // extern "C"

//@} // ingroup
#endif // end of _HAL_H_ definition

