/* --COPYRIGHT--,BSD
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//! \file   modules/resolver/src/32b/resolver.c
//! \brief  Portable C fixed point code.  These functions define the
//! \brief  resolver routines.
//!
//! (C) Copyright 2011, Texas Instruments, Inc.


// **************************************************************************
// the includes

#include "sw/modules/resolver/src/32b/resolver.h"


// **************************************************************************
// the defines


// **************************************************************************
// the globals

// **************************************************************************
// the functions

RESOLVER_Handle RESOLVER_init(void *pMemory,const size_t numBytes)
{
  RESOLVER_Handle resolverHandle;
  
  if(numBytes < sizeof(RESOLVER_Obj))
    return((RESOLVER_Handle)NULL);

  // assign the handle
  resolverHandle = (RESOLVER_Handle)pMemory;
  
  return(resolverHandle);
} // end of RESOLVER_init() function

void RESOLVER_setup(RESOLVER_Handle resolverHandle, const int16_t sample_period, const uint16_t num_pole_pairs, const uint16_t num_enc_slots, const uint32_t enc_zero_offset, const float_t full_scale_freq, const float_t speed_update_freq, const float_t speed_cutoff)
{
  RESOLVER_Obj *resolver;
  float_t temp;
  float_t speed_cutoff_radians;
  int16_t i;
 
  // create an object pointer for manipulation
  resolver = (RESOLVER_Obj *)resolverHandle;
  
  // setup the resolver sample rate
  resolver->sample_count = 0;
  resolver->sample_period = sample_period;
  
  // copy the parameters into the data structure
  resolver->num_enc_slots = num_enc_slots;
  resolver->num_pole_pairs = num_pole_pairs;
  resolver->enc_zero_offset = enc_zero_offset;
  resolver->full_scale_freq = full_scale_freq;
  resolver->delta_enc = 0;
  resolver->prev_enc = 0;
  
  // initialize the electrical angle
  resolver->enc_elec_angle = 0;
    
  // compute the gain which translates the mech into the elec angle
  resolver->mech_angle_gain = (_iq)((((uint32_t)1)<<24)/(4*num_enc_slots));

  // compute the speed gain
  temp = ((float_t)num_pole_pairs*speed_update_freq*RESOLVER_SPEED_SCALING_FACTOR) / (4.0*(float_t)num_enc_slots*full_scale_freq*(float_t)sample_period);
  resolver->speed_gain = (int32_t) temp;
  
  // compute the rpm gain
  temp = (float_t)((full_scale_freq*60.0)/num_pole_pairs);
  resolver->rpm_gain = (int32_t) temp;
  
  // compute the speed coefficients and initialize the low-pass filtered output
  resolver->speed_cutoff = speed_cutoff;
  speed_cutoff_radians = RESOLVER_2PI*speed_cutoff;
  temp = speed_cutoff_radians/(speed_update_freq+speed_cutoff_radians);
  resolver->speed_lpf_cx = (int32_t) (RESOLVER_SPEED_COEFF_SCALING*temp);
  resolver->speed_lpf_cy = RESOLVER_SPEED_COEFF_SCALING - resolver->speed_lpf_cx;
  resolver->speed_lpf_out = 0;
  
  // setup the resolver log
  resolver->log_state        = RESOLVER_LOG_STATE_IDLE;
  resolver->run_flag         = 0;
  resolver->trigger_idx      = 0;
  resolver->trigger_delta    = 0;
  resolver->log_idx          = 0;
  resolver->post_trigger_len = RESOLVER_LOG_LEN>>1;
  resolver->post_trigger_cnt = resolver->post_trigger_len;
  for (i=0; i<RESOLVER_LOG_LEN; i++)
  {
  	resolver->log[i] = 0;
  }
  
  return;
} // end of ENC setup function


void RESOLVER_calcElecAngle(RESOLVER_Handle resolverHandle, uint32_t posnCounts)
{
	RESOLVER_Obj *resolver;
	uint32_t temp;
 
  // create an object pointer for manipulation
  resolver = (RESOLVER_Obj *) resolverHandle;
	
  // compute the mechanical angle
  temp = posnCounts*resolver->mech_angle_gain;
  // add in calibrated offset
  temp += resolver->enc_zero_offset;
  // convert to electrical angle
  temp = temp * resolver->num_pole_pairs;
  // wrap around 1.0 (Q24)
  temp &= ((uint32_t) 0x00ffffff);
  // store resolver electrical angle
  resolver->enc_elec_angle = (_iq)temp;
  // update the slip angle
  resolver->enc_slip_angle = resolver->enc_slip_angle + resolver->incremental_slip;
  // wrap around 1.0 (Q24)
  resolver->enc_slip_angle &= ((uint32_t) 0x00ffffff);
  // add in compensation for slip
  temp = temp + resolver->enc_slip_angle;
  // wrap around 1.0 (Q24)
  temp &= ((uint32_t) 0x00ffffff);
  // store resolver magnetic angle
  resolver->enc_magnetic_angle = (_iq)temp;
  
  return;
} // end of RESOLVER_calc_elec_angle() function



void RESOLVER_run(RESOLVER_Handle resolverHandle, uint32_t posnCounts, uint16_t indextFlag, uint16_t dirFlag, int16_t log_flag)
{
	uint16_t dir;
	uint16_t index_event_before;
	uint16_t index_event_after;
	RESOLVER_Obj *resolver;
	int32_t enc_val;
	int32_t delta_enc;
	int32_t temp;
	int16_t sample_count;
 
  // create an object pointer for manipulation
  resolver = (RESOLVER_Obj *) resolverHandle;
  
  // update the resolver counter
  sample_count = resolver->sample_count;
  sample_count++;
  
  // if it reaches the sample period, read and process resolver data
  if (sample_count == resolver->sample_period)
  {
    resolver->sample_count = 0;
    
    // check for index event before the resolver reading
    index_event_before = indextFlag;
  	
    // read the resolver
    enc_val = posnCounts;
	
    // compute the mechanical angle
    // compute the mechanical angle
    temp = (enc_val)*resolver->mech_angle_gain;
    // add in calibrated offset
    temp += resolver->enc_zero_offset;
    // convert to electrical angle
    temp = temp * resolver->num_pole_pairs;
    // wrap around 1.0
    temp &= ((uint32_t) 0x00ffffff);

    resolver->enc_elec_angle = (_iq)temp;
  
    /*********************/
    /* compute the SPEED */
    /*********************/
  
    // read QEP direction from quadrature direction latch flag
    dir = dirFlag;
  
    // check for index event after the resolver reading
    index_event_after = indextFlag;
  
    // handle a rollover event
    if (index_event_after)
    {
  	   if (dir)
  	   {
  	     delta_enc = enc_val + (4*resolver->num_enc_slots-1) - resolver->prev_enc;
  	   }
  	   else
  	   {
         delta_enc = resolver->prev_enc + (4*resolver->num_enc_slots-1) - enc_val;
  	   }
    }
    else
    {
       delta_enc = enc_val - resolver->prev_enc;
    }
    
    // save off the delta resolver value in the data structure only if the index event before and after are the same
    if ((index_event_after == index_event_before) && (abs(delta_enc) < resolver->num_enc_slots))
    	resolver->delta_enc = delta_enc;
    
    // log the startup data
    switch(resolver->log_state)
    {
    	// wait for run flag to be set
    	case RESOLVER_LOG_STATE_IDLE:
    	{
	    	if (resolver->run_flag)
	    	{
	    		resolver->run_flag  = 0;
	    		resolver->log_idx   = 0;
	    		resolver->log_state = RESOLVER_LOG_STATE_FREERUN;
	    	}
    	}
    	break;
    	
    	// collect data round robin until there's a trigger event
    	case RESOLVER_LOG_STATE_FREERUN:
    	{
			resolver->log[resolver->log_idx] = (int16_t)enc_val;
			resolver->log_idx = resolver->log_idx + 1;
			if (resolver->log_idx > RESOLVER_LOG_LEN)
				resolver->log_idx = 0;
			if (abs(delta_enc)>RESOLVER_LOG_DELTA_TRIGGER_THRES)
			{
				resolver->trigger_delta = delta_enc;
				resolver->trigger_idx = resolver->log_idx;
				resolver->post_trigger_cnt = resolver->post_trigger_len;
				resolver->log_state = RESOLVER_LOG_STATE_ACQUIRE;
			}
    	}    		
    	break;
    	
    	// when trigger occurs collect 1/2 a buffer of post-trigger information
    	case RESOLVER_LOG_STATE_ACQUIRE:
    	{
			resolver->log[resolver->log_idx] = (int16_t)enc_val;
			resolver->log_idx = resolver->log_idx + 1;
			if (resolver->log_idx > RESOLVER_LOG_LEN)
				resolver->log_idx = 0;
			resolver->post_trigger_cnt = resolver->post_trigger_cnt - 1;
			if (resolver->post_trigger_cnt == 0)
				resolver->log_state = RESOLVER_LOG_STATE_IDLE;
    	}      	
    	break;
    }

    // shift it to Q24
    temp = resolver->delta_enc*resolver->speed_gain;
  
    // LPF the resolver
    temp = (resolver->speed_lpf_cy)*resolver->speed_lpf_out + (resolver->speed_lpf_cx)*temp;
    temp >>= RESOLVER_SPEED_COEFF_Q;
    resolver->speed_lpf_out = temp;

    // copy the current into the previous value  
    resolver->prev_enc = enc_val;
  }
  else
  {
  	resolver->sample_count = sample_count;
  }
  
} // end of RESOLVER_run() function


int16_t RESOLVER_getSpeedRPM(RESOLVER_Handle resolverHandle)
{
	RESOLVER_Obj *resolver;
	_iq temp;
 
  // create an object pointer for manipulation
  resolver = (RESOLVER_Obj *) resolverHandle;
  
  temp = (resolver->speed_lpf_out >> RESOLVER_RPM_Q1);
  temp = (temp * resolver->rpm_gain) >> RESOLVER_RPM_Q2;
	
  return (int16_t) temp;
} // end of RESOLVER_getSpeedRPM() function


// end of file
