/* --COPYRIGHT--,BSD
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#ifndef _RESOLVER_H_
#define _RESOLVER_H_

//! \file   modules/resolver/src/32b/resolver.h
//! \brief  Contains the public interface to the 
//!         resolver module routines
//!
//! (C) Copyright 2011, Texas Instruments, Inc.


// **************************************************************************
// the includes
#include "sw/modules/types/src/types.h"
#include "sw/modules/iqmath/src/32b/IQmathLib.h"

//!
//!
//! \defgroup ENC ENC
//!
//@{

#ifdef __cplusplus
extern "C" {
#endif


// **************************************************************************
// the defines
#define RESOLVER_SPEED_SCALING_FACTOR			16777216.0
#define RESOLVER_ZERO_OFFSET					3813071.0
#define RESOLVER_SPEED_CUTOFF_FREQ				100.0
#define RESOLVER_SPEED_COEFF_Q					6
#define RESOLVER_SPEED_COEFF_SCALING			(1<<RESOLVER_SPEED_COEFF_Q)
#define RESOLVER_2PI							(2.0*3.14159)
#define RESOLVER_LOG_LEN						256
#define RESOLVER_RPM_Q1							8
#define RESOLVER_RPM_Q2							16
#define RESOLVER_LOG_DELTA_TRIGGER_THRES		400


// **************************************************************************
// the typedefs

//! \brief Enumeration for the resolver log states
//!
typedef enum {
  RESOLVER_LOG_STATE_IDLE=0,            //!< idle state
  RESOLVER_LOG_STATE_FREERUN,	        //!< freerun state
  RESOLVER_LOG_STATE_ACQUIRE	        //!< acquire state
} RESOLVER_LOG_State_e;


//! \brief Defines the resolver object
//!
typedef struct _RESOLVER_Obj_
{
  int16_t sample_count;			    //!< when it reaches the sample period, collect & process resolver data
  int16_t sample_period;		    //!< sample period of resolver processing
  uint16_t num_pole_pairs;		    //!< number of pole pairs in motor
  uint16_t num_enc_slots;		    //!< number of resolver slots
  _iq mech_angle_gain;			    //!< gain which converts the resolver counts to Q24 mechanical degrees
  uint32_t enc_zero_offset;         //!< resolver zero offset in counts
  int32_t enc_elec_angle;		    //!< resolver current electrical angle
  int32_t incremental_slip;         //!< incremental amount of slip
  int32_t enc_slip_angle;           //!< amount of total slip
  int32_t enc_magnetic_angle;       //!< resolver current magnetic angle (compensated for slip)
  int32_t prev_enc;				    //!< previous resolver reading
  int32_t delta_enc;			    //!< resolver count delta
  float_t full_scale_freq;	        //!< full scale frequency
  int32_t speed_gain;			    //!< gain which converts a difference in resolver counts to Q24 normalized electrical freq
  int32_t rpm_gain;				    //!< gain which converts the Q24 normalized electrical freq to RPM
  float_t speed_cutoff;		        //!< speed cutoff frequency in Hz
  int32_t speed_lpf_cx;			    //!< speed input coefficient
  int32_t speed_lpf_cy;			    //!< speed output coefficient
  int32_t speed_lpf_out;		    //!< speed lpf output
  RESOLVER_LOG_State_e log_state;	//!< resolver log state
  int16_t run_flag;				    //!< resolver log free run flag
  int16_t post_trigger_len;		    //!< resolver log post trigger length
  int16_t post_trigger_cnt;		    //!< resolver log post trigger counter
  int16_t trigger_idx;			    //!< index where trigger event happened
  int16_t trigger_delta;		    //!< calculated delta when trigger happened
  int32_t log_idx;			        //!< resolver log index
  int16_t log[RESOLVER_LOG_LEN];     //!< resolver log length
} RESOLVER_Obj;


//! \brief Defines the resolver handle
//!
typedef struct _RESOLVER_Obj_ *RESOLVER_Handle;


// **************************************************************************
// the function prototypes

//! \brief Reads resolver and returns the electrical degrees in Q24 format
//! \param[in] resolverHandle			Handle to the ENC object
//! \param[in] posnCounts               Current position counts from resolver
//! \return								Nothing
void RESOLVER_calcElecAngle(RESOLVER_Handle resolverHandle, uint32_t posnCounts);


//! \brief Returns the electrical angle
//! \param[in] resolverHandle			Handle to the ENC object
//! \return								Electrical angle in Q24
inline _iq RESOLVER_getElecAngle(RESOLVER_Handle resolverHandle) {
	RESOLVER_Obj *resolver = (RESOLVER_Obj *) resolverHandle;

	return resolver->enc_elec_angle;
}


//! \brief Returns the magnetic angle
//! \param[in] resolverHandle			Handle to the ENC object
//! \return								Magnetic angle in Q24
inline _iq RESOLVER_getMagneticAngle(RESOLVER_Handle resolverHandle) {
	RESOLVER_Obj *resolver = (RESOLVER_Obj *) resolverHandle;

	return resolver->enc_magnetic_angle;
}


//! \brief Returns the low-pass filtered speed output
//! \param[in] resolverHandle			Handle to the ENC object
//! \return								LPF speed output in Q24
inline _iq RESOLVER_getFilteredSpeed(RESOLVER_Handle resolverHandle) {
	RESOLVER_Obj *resolver = (RESOLVER_Obj *) resolverHandle;

	return resolver->speed_lpf_out;
}


//! \brief Returns the filtered speed in RPM
//! \param[in] resolverHandle			Handle to the ENC object
//! \return								RPM in Q0
extern int16_t RESOLVER_getSpeedRPM(RESOLVER_Handle resolverHandle);


//! \brief Initializes the resolver object
//! \param[in] pMemory		Memory pointer to object
//! \param[in] numBytes		Object size
//! \return					Object handle
extern RESOLVER_Handle RESOLVER_init(void *pMemory,const size_t numBytes);


//! \brief Based on the resolver reading, computes the electrical angle and the electrical "speed"
//! \param[in] resolverHandle			Handle to the ENC object
//! \param[in] posnCounts               Current position counts from resolver
//! \param[in] indextFlag               If set, there was an index
//! \param[in] dirFlag                  Indicates direction of rotation
//! \param[in] log_flag					If set, logs the resolver data
//! \return								Nothing
void RESOLVER_run(RESOLVER_Handle resolverHandle, uint32_t posnCounts, uint16_t indextFlag, uint16_t dirFlag, int16_t log_flag);


//! \brief Set the amount of incremental slip
//! \param[in] resolverHandle			Handle to the ENC object
//! \param[in] incrementalSlip          Amount of incremental slip in Electrical Angle
inline void RESOLVER_setIncrementalSlip(RESOLVER_Handle resolverHandle, _iq incrementalSlip) {
	RESOLVER_Obj *resolver = (RESOLVER_Obj *) resolverHandle;

	// set the amount of incremental slip
	resolver->incremental_slip = incrementalSlip;

	return;
}


//! \brief Start logging resolver data
//! \param[in] resolverHandle			Handle to the ENC object
inline void RESOLVER_setRunFlag(RESOLVER_Handle resolverHandle) {
	RESOLVER_Obj *resolver = (RESOLVER_Obj *) resolverHandle;

	// set the run flag
	resolver->run_flag = 1;

	return;
}


//! \brief Sets the value for the resolver object zero offset
//! \param[in] resolverHandle           Handle to the ENC object
//! \param[in] zeroOffset               New zero offset
inline void RESOLVER_setZeroOffset(RESOLVER_Handle resolverHandle, uint32_t zeroOffset) {
	RESOLVER_Obj *resolver = (RESOLVER_Obj *)resolverHandle;

	resolver->enc_zero_offset = zeroOffset*resolver->mech_angle_gain;

	return;
}


//! \brief Initializes resolver object parameters
//! \param[in] resolverHandle                   Handle to the ENC object
//! \param[in] sample_period                    How often the resolver is read & processed
//! \param[in] num_pole_pairs                   Number of pole pairs in motor
//! \param[in] num_enc_slots                    Number of resolver slots
//! \param[in] enc_zero_offset                  Encoder zero offset in counts
//! \param[in] full_scale_freq                  Full scale speed for normalization
//! \param[in] speed_update_freq                Update frequency in Hz for speed calculation
//! \param[in] speed_cutoff                     Speed calculation LPF cutoff frequency in Hz
extern void RESOLVER_setup(RESOLVER_Handle resolverHandle, const int16_t sample_period,
                      const uint16_t num_pole_pairs, const uint16_t num_enc_slots, 
                      const uint32_t enc_zero_offset, const float_t full_scale_freq, 
                      const float_t speed_update_freq, const float_t speed_cutoff);

#ifdef __cplusplus
}
#endif // extern "C"

//@} // ingroup
#endif // end of _RESOLVER_H_ definition

