/******************************************************************************
* TIDA-01507 Resolver-to-Uart functionality only (based on TIDA-00796)
* Module: PGA411 driver
* Author: Jiri Panacek, j-panacek@ti.com
*
* Copyright 2017 Texas Instruments Incorporated. All rights reserved.
******************************************************************************/
#include "F2806x_Device.h"

#include "sw/safety/pga411q1/src/32b/f28x/f2806x/pga411q1.h"
#include "sw/safety/pga411q1/src/32b/f28x/f2806x/pga411q1_spi.h"

/******************************************************************************
 * MACROS
 *****************************************************************************/

#define    CRC_BITCOUNT     24      // top 24 bits of Uint32 for CRC6
#define    CRC_BYTECOUNT    3       // top 3 B of Uint32 for CRC6
#define    CRC_INITSEED     0x3F    // initial seed for CRC6
#define    CRC_POLYNOM      0x5B    // CRC = X6+X4+X3+X+1
#define    SPI_DUMMY        0xF0    // dummy SPI data for read operation
#define    EMPTY_LOOP

PGA411Q1_Vars_t gPga411q1Vars;

/******************************************************************************
 * ENUMS
 *****************************************************************************/

/******************************************************************************
 * TYPEDEFS
 *****************************************************************************/

/******************************************************************************
 * VARIABLES, CONSTANTS
 *****************************************************************************/

volatile pga411_spi_frame_t pga411_spi_frame = {.addr = 0x00,
                                                .dataout = 0x0000,
                                                .reserved = 0x0,
                                                .mcrc = 0x00
                                               };

pga411_reg_t pga411_regs[PGA411_REG_COUNT + 1] =
{
    /* memory constants init */
    [AD_DEV_OVUV1]      = {.read_add = 0x53, .write_add = 0x87, .def_val = 0x8B40},
    [AD_DEV_OVUV2]      = {.read_add = 0x6B, .write_add = 0x26, .def_val = 0x00ED},
    [AD_DEV_OVUV3]      = {.read_add = 0x65, .write_add = 0x17, .def_val = 0xFCFF},
    [AD_DEV_OVUV4]      = {.read_add = 0xEC, .write_add = 0x39, .def_val = 0x07F2},
    [AD_DEV_OVUV5]      = {.read_add = 0x52, .write_add = 0x75, .def_val = 0x1C00},
    [AD_DEV_OVUV6]      = {.read_add = 0xE9, .write_add = 0x83, .def_val = 0x038F},
    [AD_DEV_TLOOP_CFG]  = {.read_add = 0xA6, .write_add = 0x42, .def_val = 0x0514},
    [AD_DEV_AFE_CFG]    = {.read_add = 0xC2, .write_add = 0x91, .def_val = 0x0005},
    [AD_DEV_PHASE_CFG]  = {.read_add = 0x57, .write_add = 0x85, .def_val = 0x1400},
    [AD_DEV_CONFIG1]    = {.read_add = 0xBE, .write_add = 0xEB, .def_val = 0x0002},
    [AD_DEV_CONTROL1]   = {.read_add = 0x90, .write_add = 0x0D, .def_val = 0x0000},
    [AD_DEV_CONTROL2]   = {.read_add = 0x63, .write_add = 0x38, .def_val = 0x0000},
    [AD_DEV_CONTROL3]   = {.read_add = 0xDD, .write_add = 0xAE, .def_val = 0x0003},
    [AD_DEV_STAT1]      = {.read_add = 0x81, .write_add = 0x00, .def_val = 0x0000},
    [AD_DEV_STAT2]      = {.read_add = 0x4D, .write_add = 0x00, .def_val = 0x0000},
    [AD_DEV_STAT3]      = {.read_add = 0x84, .write_add = 0x00, .def_val = 0x0000},
    [AD_DEV_STAT4]      = {.read_add = 0x1F, .write_add = 0x00, .def_val = 0x0000},
    [AD_DEV_STAT5]      = {.read_add = 0x41, .write_add = 0x00, .def_val = 0x0000},
    [AD_DEV_STAT6]      = {.read_add = 0x6F, .write_add = 0x00, .def_val = 0x0000},
    [AD_DEV_STAT7]      = {.read_add = 0xE1, .write_add = 0x00, .def_val = 0x0000},
    [AD_DEV_CLCRC]      = {.read_add = 0x4F, .write_add = 0xFC, .def_val = 0x0000},
    [AD_DEV_CRC]        = {.read_add = 0x0F, .write_add = 0xE7, .def_val = 0x0000},
    [AD_CRCCALC]        = {.read_add = 0xD9, .write_add = 0xFF, .def_val = 0x0000},
    [AD_DEV_EE_CTRL1]   = {.read_add = 0xE3, .write_add = 0x6E, .def_val = 0x0000},
    [AD_DEV_CRC_CTRL1]  = {.read_add = 0x7A, .write_add = 0xB6, .def_val = 0x0000},
    [AD_DEV_EE_CTRL4]   = {.read_add = 0xBA, .write_add = 0x56, .def_val = 0x0000},
    [AD_DEV_UNLK_CTRL1] = {.read_add = 0x64, .write_add = 0x95, .def_val = 0x0000},
    /* for continuous read we need to read 28 times. This significantly helps */
    [AD_DUMMY_REG]      = {.read_add = 0x53, .write_add = 0x87, .def_val = 0x0000},
};

/******************************************************************************
 * INTERNAL FUNCTIONS PROTOTYPES
 *****************************************************************************/

/* calculate CRC6 for output data, function takes first 3B left aligned data */
uint16_t pga411_crc2(Uint32 datin);

/* Xmit data to PGA over SPI, for reg use defined macros, return incoming frame */
pga411_spi_frame_t pga411_XmitSPI(uint16_t dir, uint16_t reg, uint16_t wdata);

/******************************************************************************
 * INTERNAL FUNCTIONS
 *****************************************************************************/

/* CRC6 calculation - optimized for PGA411 */
uint16_t pga411_crc2(uint32_t datin)
{
    uint16_t byte_idx, bit_idx, crc = (CRC_INITSEED << 2);

    /* byte by byte starting from most significant (3-2-1) */
    for (byte_idx = CRC_BYTECOUNT; byte_idx >= 1; byte_idx--)
    {
        /* XOR-in new byte from left to right */
        crc ^= ((datin >> (byte_idx << 3)) & 0x000000FF);

        /* bit by bit for each byte */
        for (bit_idx = 0; bit_idx < 8; bit_idx++)
        {
            crc = crc << 1 ^ (crc & 0x80 ? (CRC_POLYNOM << 2) : 0);
        }
    }

    return (crc >> 2 & CRC_INITSEED); /*restore two bit offset */
}

/******************************************************************************
 * SYSTEM INTERFACE FUNCTIONS
 *****************************************************************************/

/* Initialization PGA411-Q1 related hardware */
void pga411_Init(void)
{
    pga411_InitGPIO();      /* init GPIO */
    pga411_InitSPI();       /* init SPI periphery */
    pga411_Reset();         /* perform PGA411 reset */

    gSpiMode = SPI_PGA411;
}

/* Perform PGA411-Q1 reset using GPIO */
void pga411_Reset(void)
{
    HAL_PGA_RESET_N_LOW;    /* RESET_N low */
    usDelay(500);           /* hold in reset for 500us */
    HAL_PGA_RESET_N_HIGH;   /* release from reset */
    usDelay(10000);    /* wait at least 10 ms*/
}

/* Perform PGA411-Q1 FAULT reset using GPIO */
void pga411_FaultReset(void)
{
    HAL_PGA_FAULTRES_LOW; /* assert FAULTRES low - open collector */
    usDelay(1000); /* hold in the state for 1ms */
    HAL_PGA_FAULTRES_HIZ; /* return back to high-z*/
}
/* Xmit data to PGA over SPI, for reg use defined macros */
pga411_spi_frame_t pga411_XmitSPI(uint16_t dir, uint16_t reg, uint16_t wdata)
{
    pga411_spi_frame_t out, in;

    /* do we read data ? */
    if (dir == READ) { out.addr = pga411_regs[reg].read_add; } /* read address */
    /* or write data ? */
    else { out.addr = pga411_regs[reg].write_add; } /* write address */

    /* compose the rest of the frame */
    out.dataout = wdata; /* real data (W) or dummy data (R) */
    out.reserved = 0x00; /* always zero */
    out.mcrc = pga411_crc2(out.frame); /* calculate TX CRC */
    in.frame = pga411_Xmit4BSPI(out.frame);

    /* check RX CRC */
    if (pga411_crc2(in.frame) != in.scrc)
    {
        pga411_assert(); /* if error -> terminate */
    }

    return (in);
}

/* Read data from defined register */
uint16_t pga411_ReadReg(uint16_t reg)
{
    /* first read returns whatever */
    pga411_XmitSPI(READ, reg, SPI_DUMMY);

    /* second read returns requested data */
    return (pga411_XmitSPI(READ, reg, SPI_DUMMY).datain);
}

/* Write data to defined register */
void pga411_WriteReg(uint16_t reg, uint16_t data)
{
    /* here we just making it nice, can be macro too */
    pga411_XmitSPI(WRITE, reg, data);
}

/* Read all registers from PGA */
void pga411_ReadAll(void)
{
    int i;
    uint16_t rx_data;

    /* register by register (27+1) */
    /* PGA411 always returns previous read register value */
    /* thats why 28 reads are needed (first read returns whatever data) */
    for (i = 0; i < PGA411_REG_COUNT + 1; i++)
    {
        rx_data = pga411_XmitSPI(READ, i, SPI_DUMMY).datain;

        /*boolean trick - save data if not first cycle */
        if (i)
        {
            pga411_regs[i - 1].real_val = rx_data;
        }
    }
}

/* Change state diagnostic/normal */
void pga411_State(uint16_t state)
{
    uint16_t temp;

    /* Enter Diagnostic state */
    if (state == DIAG)
    {
        /* read content of  DEV_CONTROL3 register */
        temp = pga411_ReadReg(AD_DEV_CONTROL3);
        temp |= 0x0004; /* set bit SPIDIAG */
        pga411_WriteReg(AD_DEV_CONTROL3, temp); /* finish R-M-W */
    }
    /* Go back to normal state */
    else
    {
        /* read content of  DEV_CONTROL1 register */
        temp = pga411_ReadReg(AD_DEV_CONTROL1);
        temp |= 0x0001; /* set bit DIAGEXIT */
        pga411_WriteReg(AD_DEV_CONTROL1, temp); /* finish R-M-W */
    }
}

/* Device unlock (must be in diagnostic state) */
void pga411_DeviceUnlock(void)
{
    pga411_WriteReg(AD_DEV_UNLK_CTRL1, 0x000F);
    pga411_WriteReg(AD_DEV_UNLK_CTRL1, 0x0055);
    pga411_WriteReg(AD_DEV_UNLK_CTRL1, 0x00AA);
    pga411_WriteReg(AD_DEV_UNLK_CTRL1, 0x00F0);
}

/* EEPROM unlock (must be in diagnostic state) */
void pga411_EEPROMUnlock(void)
{
    pga411_WriteReg(AD_DEV_EE_CTRL4, 0x000F);
    pga411_WriteReg(AD_DEV_EE_CTRL4, 0x0055);
    pga411_WriteReg(AD_DEV_EE_CTRL4, 0x00AA);
    pga411_WriteReg(AD_DEV_EE_CTRL4, 0x00F0);
}

/* Write default configuration to PGA */
void pga411_DefaultCfg(PGA411Q1_Vars_t *pga411q1)
{
    int i;

    pga411q1->readAngle = false;
    pga411q1->readVelocity = false;
    pga411q1->readCmd = false;
    pga411q1->readStat = false;
    pga411q1->writeCmd = false;

    pga411q1->angleOffset = _IQ(0.1625731587);
    pga411q1->angle = _IQ(0.0);
    pga411q1->angle_pu = _IQ(0.0);
    pga411q1->velocity_pu = _IQ(0.0);
    pga411q1->velocityCoef = 20000000/1000/2;

    /* go to diagnostic state */
    pga411_State(DIAG);

    /* unlock registers */
    pga411_DeviceUnlock();

    /* first 12 registers to default */
    /* note: for some u need to unlocked them first ! */
    for (i = 0; i < 12; i++)
    {
        pga411_XmitSPI(WRITE, i, pga411_regs[i].def_val);
    }

    /* go to normal state */
    pga411_State(NORMAL);

    pga411_ReadAll();

    pga411q1->DEV_TLOOP_CFG.all = pga411_regs[AD_DEV_TLOOP_CFG].real_val;
    pga411q1->DEV_AFE_CFG.all = pga411_regs[AD_DEV_AFE_CFG].real_val;
    pga411q1->DEV_PHASE_CFG.all = pga411_regs[AD_DEV_PHASE_CFG].real_val;
    pga411q1->DEV_CONFIG1.all = pga411_regs[AD_DEV_CONFIG1].real_val;
    pga411q1->DEV_CONTROL1.all = pga411_regs[AD_DEV_CONTROL1].real_val;
    pga411q1->DEV_CONTROL2.all = pga411_regs[AD_DEV_CONTROL2].real_val;
    pga411q1->DEV_CONTROL3.all = pga411_regs[AD_DEV_CONTROL3].real_val;
}

/* Function reads angle from the PGA411 and sends it over UART */
void pga411_GetAngle(PGA411Q1_Vars_t *pga411q1)
{
    if(pga411q1->readAngle == true)
    {
        if(gSpiMode != SPI_PGA411)
        {
            pga411_InitSPI();
            gSpiMode = SPI_PGA411;
        }

        /* angle calculation for the PGA411-Q1 using IQmathLib library */
        /* _iq19 stores -4096 to 4095.999998093 with 0.000001907 resolution */
        uint16_t angle_raw; /* raw data from the PGA411-Q1 register */
        _iq angle; /* IQ math angle */

        /* read data from the PGA411-Q1 */
        angle_raw = pga411_ReadReg(AD_DEV_STAT5);
        pga411q1->DEV_STAT5.all = angle_raw;

        angle_raw &= 0x1FFF; /* preserve only ORDANGLE bits */
        /* multiply minimal step (LSB) with ORDANGLE value */
        pga411q1->angle = ((uint32_t)angle_raw)*4096;         // 12b example, IQ12->IQ24

        angle = pga411q1->angle - pga411q1->angleOffset;

        if(angle < _IQ(0.0))
            angle = angle + _IQ(1.0);

        pga411q1->angle_pu = angle;
        pga411q1->readAngle = false;
    }

    return;
}

/* Function reads velocity from the PGA411 and sends it over UART */
void pga411_GetVelocity(PGA411Q1_Vars_t *pga411q1)
{
    if(pga411q1->readVelocity == true)
    {
        if(gSpiMode != SPI_PGA411)
        {
            pga411_InitSPI();
            gSpiMode = SPI_PGA411;
        }

        /* velocity calculation for the PGA411-Q1 using IQmathLib library */
        /* _iq14 stores -131072 to 131071.999938965 with 0.000061035 resolution */
        int16_t velocity_raw; /* raw data from the PGA411-Q1 register */
        _iq velocity; /* IQ math velocity */

        /* read data from the PGA411-Q1 */
        velocity_raw = pga411_ReadReg(AD_DEV_STAT6);
        pga411q1->DEV_STAT6.all = velocity_raw;

        /* convert 2nd complement to float */
        if (velocity_raw & 0x0800) /* negative number ? */
        {
            /* convert to positive number first */
            velocity = (((~velocity_raw) + 1) & 0x07FF);
            velocity = -velocity; /* and make float negative */
        }
        else
        {
            /* positive number, preserve only needed bits */
            velocity = (velocity_raw & 0x07FF);
        }

        pga411q1->velocity_pu = velocity*pga411q1->velocityCoef;

        pga411q1->readVelocity = false;
    }

    return;
}
