#ifndef _PGA411Q1_H_
#define _PGA411Q1_H_
/******************************************************************************
* TIDA-01507 Resolver-to-Uart functionality only (based on TIDA-00796)
* Module: PGA411 driver
* Author: Jiri Panacek, j-panacek@ti.com
*
* Copyright 2017 Texas Instruments Incorporated. All rights reserved.
******************************************************************************/
#include "F2806x_Device.h"     // F2806x Headerfile Include File

#include "sw/modules/types/src/types.h"
#include "sw/modules/usDelay/src/32b/usDelay.h"
#include "sw/modules/iqmath/src/32b/IQmathLib.h"

#include "sw/safety/com_spi/src/32b/f28x/com_spi.h"
#include "sw/safety/pga411q1/src/32b/f28x/f2806x/pga411q1_spi.h"

/******************************************************************************
 * MACROS
 *****************************************************************************/

/* PGA411-Q1 registers and their offsets */
#define     AD_DEV_OVUV1       0x00
#define     AD_DEV_OVUV2       0x01
#define     AD_DEV_OVUV3       0x02
#define     AD_DEV_OVUV4       0x03
#define     AD_DEV_OVUV5       0x04
#define     AD_DEV_OVUV6       0x05
#define     AD_DEV_TLOOP_CFG   0x06
#define     AD_DEV_AFE_CFG     0x07
#define     AD_DEV_PHASE_CFG   0x08
#define     AD_DEV_CONFIG1     0x09
#define     AD_DEV_CONTROL1    0x0A /* device unlock needed */
#define     AD_DEV_CONTROL2    0x0B /* device unlock needed */
#define     AD_DEV_CONTROL3    0x0C
#define     AD_DEV_STAT1       0x0D
#define     AD_DEV_STAT2       0x0E
#define     AD_DEV_STAT3       0x0F
#define     AD_DEV_STAT4       0x10
#define     AD_DEV_STAT5       0x11
#define     AD_DEV_STAT6       0x12
#define     AD_DEV_STAT7       0x13
#define     AD_DEV_CLCRC       0x14
#define     AD_DEV_CRC         0x15
#define     AD_CRCCALC         0x16
#define     AD_DEV_EE_CTRL1    0x17 /*EEPROM unlock needed */
#define     AD_DEV_CRC_CTRL1   0x18
#define     AD_DEV_EE_CTRL4    0x19
#define     AD_DEV_UNLK_CTRL1  0x1A
#define     AD_DUMMY_REG       0x1B /* this simplifies continuous read */

#define     PGA411_REG_COUNT    27

#define     DATA_DUMMY          0x00 /* dummy for SPI read */

/******************************************************************************
 * ENUMS
 *****************************************************************************/

enum {READ, WRITE}; /* Enums selects SPI operation */

enum {DIAG, NORMAL}; /* Operational states of PGA */

/******************************************************************************
 * TYPEDEFS
 *****************************************************************************/

typedef union
{
    /* outgoing data frame (from master to slave) */
    struct
    {
        /* reverse order in bit-fields, starting from bit 0 */
        uint32_t    mcrc: 6;      /* master's CRC for data, bits 0..5 */
        uint32_t    reserved: 2;  /* reserved, always 0, bits 6..7  */
        uint32_t    dataout: 16;  /* data, MSB first, bits 8..23 */
        uint32_t    addr: 8;      /* register address, bits 24..31 */
    };
    /* incomming data frame (to master from slave */
    struct
    {
        uint32_t    scrc: 6;      /* slave's CRC for data, bits 0..5 */
        uint32_t    stat: 2;      /* status of SPI communication, bits 6..7 */
        uint32_t    datain: 16;   /* data, MSB first, bits 8..23 */
        uint32_t    ecbk: 8;      /* register address, bits 24..31 */
    };
    /* and finaly the whole frame */
    struct
    {
        uint32_t    frame;
    };
} pga411_spi_frame_t;

typedef struct
{
    /* registers definition */
    const uint16_t read_add;   /* SPI read address */
    const uint16_t write_add;  /* SPI write address */
    const uint16_t def_val;    /* default PGA value */
    uint16_t real_val;         /* real-time register content */
} pga411_reg_t;

typedef struct
{
    // reverse order in bit-fields, starting from bit 0
    uint32_t    crc: 6;        // master's CRC for data, bits 0..5
    uint32_t    stat: 2;        // reserved, always 0, bits 6..7
    uint32_t    data: 16;       // data, MSB first, bits 8..23
    uint32_t    addr: 8;        // register address, bits 24..31
} PGA411Q1_Bytes_t;

typedef struct
{
    // reverse order in bit-fields, starting from bit 0
    uint16_t    dataH;          //
    uint16_t    dataL;          //
} PGA411Q1_Words_t;

typedef union
{
    // data frame
    PGA411Q1_Bytes_t   AddrData;
    PGA411Q1_Words_t   dataWord;
    // and finaly the whole frame
    uint32_t           frame;
} PGA411Q1_spi_frame_t;

typedef enum
{
    PGA_READ = 0,                  //!< SPI read operation
    PGA_WRITE = 1                  //!< SPI write operation
} PGA_CMD_e;

typedef enum
{
    PGA_DIAG = 0,                  //!< Operational states of PGA
    PGA_NORMAL = 1                 //!< Operational states of PGA
} PGA_STATE_e;

typedef enum
{
    H0p525V = 0,                    //!< 0.525V
    H0p550V = 1,                    //!< 0.550V
    H0p575V = 2,                    //!< 0.575V
    H0p600V = 3,                    //!< 0.600V
    H0p625V = 4,                    //!< 0.625V
    H0p650V = 5,                    //!< 0.650V
    H0p675V = 6,                    //!< 0.675V
    H0p700V = 7                     //!< 0.700V
} OSHORTH_e;

typedef enum
{
    L0p475V = 0,                    //!< 0.475V
    L0p450V = 1,                    //!< 0.450V
    L0p425V = 2,                    //!< 0.425V
    L0p400V = 3,                    //!< 0.400V
    L0p375V = 4,                    //!< 0.375V
    L0p350V = 5,                    //!< 0.350V
    L0p325V = 6,                    //!< 0.325V
    L0p300V = 7                     //!< 0.300V
} OSHORTL_e;

typedef enum
{
    H150mA = 0,                    //!< 150mA
    H165mA = 1,                    //!< 165mA
    H180mA = 2,                    //!< 180mA
    H200mA = 3,                    //!< 200mA
    H225mA = 4,                    //!< 225mA
    H260mA = 5,                    //!< 260mA
    H300mA = 6,                    //!< 300mA
    H360mA = 7                     //!< 360mA
} EXTILIMTH_H1_2_e;

typedef enum
{
    L140mA = 0,                    //!< 140mA
    L150mA = 1,                    //!< 150mA
    L165mA = 2,                    //!< 165mA
    L180mA = 3,                    //!< 180mA
    L200mA = 4,                    //!< 200mA
    L230mA = 5,                    //!< 230mA
    L550mA1 = 6,                   //!< 550mA/N
    L550mA2 = 7                    //!< 550mA/N
} EXTILIMTH_L1_2_e;

typedef enum
{
    G1p15 = 0,                    //!< Gain=1.15
    G1p20 = 1,                    //!< Gain=1.20
    G1p25 = 2,                    //!< Gain=1.25
    G1p30 = 3,                    //!< Gain=1.30
    G1p35 = 4,                    //!< Gain=1.35
    G1p40 = 5,                    //!< Gain=1.40
    G1p45 = 6,                    //!< Gain=1.45
    G1p50 = 7,                    //!< Gain=1.50
    G1p55 = 8,                    //!< Gain=1.55
    G1p60 = 9,                    //!< Gain=1.60
    G1p65 = 10,                   //!< Gain=1.65
    G1p70 = 11,                   //!< Gain=1.70
    G1p75 = 12,                   //!< Gain=1.75
    G1p80 = 13,                   //!< Gain=1.80
    G1p85 = 14,                   //!< Gain=1.85
    G1p90 = 15                    //!< Gain=1.90
} EXTOUT_GL_e;

typedef struct
{                                   // bit      description
    OSHORTH_e OSHORTH:3;                // 2~0
    OSHORTL_e OSHORTL:3;                // 5~3
    EXTILIMTH_H1_2_e EXTILIMTH_H1_2:3;  // 8~6
    EXTILIMTH_L1_2_e EXTILIMTH_L1_2:3;  // 11~9
    EXTOUT_GL_e EXTOUT_GL:4;            // 15~12
} DEV_OVUV1_bits;

typedef union  {
   uint16_t                 all;
   DEV_OVUV1_bits           bit;
} DEV_OVUV1_t;

typedef enum
{
    L2p40V = 0,                    //!< Threshold=2.40V
    L2p25V = 1,                    //!< Threshold=2.25V
    L2p10V = 2,                    //!< Threshold=2.10V
    L1p95V = 3,                    //!< Threshold=1.95V
    L1p80V = 4,                    //!< Threshold=1.80V
    L1p65V = 5,                    //!< Threshold=1.65V
    L1p50V = 6,                    //!< Threshold=1.50V
    L1p35V = 7                     //!< Threshold=1.35V
} DVMSENL_e;

typedef enum
{
    H2p60V = 0,                    //!< Threshold=2.60V
    H2p75V = 1,                    //!< Threshold=2.75V
    H2p90V = 2,                    //!< Threshold=2.90V
    H3p05V = 3,                    //!< Threshold=3.05V
    H3p20V = 4,                    //!< Threshold=3.20V
    H3p35V = 5,                    //!< Threshold=3.35V
    H3p50V = 6,                    //!< Threshold=3.50V
    H3p65V = 7                     //!< Threshold=3.65V
} DVMSENH_e;

typedef enum
{
    T90or2ms = 0,                   //!< 90ms/amode=L,  2ms/amode=H
    T120or4ms = 1,                  //!< 120ms/amode=L, 4ms/amode=H
    T150or6ms = 2,                  //!< 150ms/amode=L, 6ms/amode=H
    T180or8ms = 3                   //!< 180ms/amode=L, 8ms/amode=H
}  TRDHL_e;

typedef enum
{
    AMP_INT = 0,                    //!< the internal exciter amplifier
    AMP_EXT = 1                     //!< an external exciter amplifier
}  XEXT_AMP_e;


typedef struct
{                                   // bit      description
    DVMSENL_e DVMSENL:3;                // 2~0
    DVMSENH_e DVMSENH:3;                // 5~3
    TRDHL_e TRDHL:2;                    // 7~6
    XEXT_AMP_e XEXT_AMP:1;              // 8
    uint16_t RSVH:7;                    // 15~9
} DEV_OVUV2_bits;

typedef union  {
   uint16_t                 all;
   DEV_OVUV2_bits           bit;
} DEV_OVUV2_t;

typedef enum
{
    H0p750V = 0,                    //!< Threshold=0.750V
    H0p775V = 1,                    //!< Threshold=0.775V
    H0p800V = 2,                    //!< Threshold=0.800V
    H0p825V = 3,                    //!< Threshold=0.825V
    H0p850V = 4,                    //!< Threshold=0.850V
    H0p875V = 5,                    //!< Threshold=0.875V
    H0p900V = 6,                    //!< Threshold=0.900V
    H0p925V = 7                     //!< Threshold=0.925V
} OOPENTHH_e;

typedef enum
{
    H0p250V = 0,                    //!< Threshold=0.250V
    H0p225V = 1,                    //!< Threshold=0.225V
    H0p200V = 2,                    //!< Threshold=0.200V
    H0p175V = 3,                    //!< Threshold=0.175V
    H0p150V = 4,                    //!< Threshold=0.150V
    H0p125V = 5,                    //!< Threshold=0.125V
    H0p110V = 6,                    //!< Threshold=0.100V
    H0p075V = 7                     //!< Threshold=0.075V
} OOPENTHL_e;

typedef enum
{
    H0p75V = 0,                    //!< Threshold=0.75V
    H0p80V = 1,                    //!< Threshold=0.80V
    H0p85V = 2,                    //!< Threshold=0.85V
    H0p90V = 3                     //!< Threshold=0.90V
} OVIZH_e;

typedef enum
{
    L0p25V = 0,                    //!< Threshold=0.25V
    L0p20V = 1,                    //!< Threshold=0.20V
    L0p15V = 2,                    //!< Threshold=0.15V
    L0p10V = 3                     //!< Threshold=0.10V
} OVIZL_e;

typedef enum
{
    O1p2us = 0,                    //!<  Deglitch=1.2us
    O3p2us = 1,                    //!<  Deglitch=3.2us
    O5p2us = 2,                    //!<  Deglitch=5.2us
    O7p2us = 3,                    //!<  Deglitch=7.2us
    O9p2us = 4,                    //!<  Deglitch=9.2us
    O11p2us = 5,                   //!<  Deglitch=11.2us
    O13p2us = 6,                   //!<  Deglitch=13.2us
    O15p2us = 7                    //!<  Deglitch=15.2us
} EXTOVT_e;

typedef enum
{
    U1p2us = 0,                    //!<  Deglitch=1.2us
    U3p2us = 1,                    //!<  Deglitch=3.2us
    U5p2us = 2,                    //!<  Deglitch=5.2us
    U7p2us = 3,                    //!<  Deglitch=7.2us
    U9p2us = 4,                    //!<  Deglitch=9.2us
    U11p2us = 5,                   //!<  Deglitch=11.2us
    U13p2us = 6,                   //!<  Deglitch=13.2us
    U15p2us = 7                    //!<  Deglitch=15.2us
} EXTUVT_e;

typedef struct
{                                   // bit      description
    OOPENTHL_e OOPENTHH:3;             // 2~0
    OOPENTHL_e OOPENTHL:3;             // 5~3
    OVIZH_e OVIZH:2;                   // 7~6
    OVIZL_e OVIZL:2;                   // 9~8
    EXTOVT_e EXTOVT:3;                 // 12~10
    EXTUVT_e EXTUVT:3;                 // 15~13
} DEV_OVUV3_bits;

typedef union  {
   uint16_t                 all;
   DEV_OVUV3_bits           bit;
} DEV_OVUV3_t;

typedef struct
{                                   // bit      description
    uint16_t FSHORT_CFG:1;           // 0
    uint16_t nBOOST_FF:1;            // 1
    uint16_t VEXT_CFG:1;             // 2
    uint16_t AUTOPHASE_CFG:2;        // 4~3
    uint16_t TEXTMO:3;               // 7~5
    uint16_t TSHORT:3;               // 10~8
    uint16_t RSVH:5;                 // 15~11
} DEV_OVUV4_bits;

typedef union  {
   uint16_t                 all;
   DEV_OVUV4_bits           bit;
} DEV_OVUV4_t;

typedef struct
{                                   // bit      description
    uint16_t RSV1:10;                // 9~0
    uint16_t TOPEN:3;                // 12~10
    uint16_t RSVH:3;                 // 15~12
} DEV_OVUV5_bits;

typedef union  {
   uint16_t                 all;
   DEV_OVUV5_bits           bit;
} DEV_OVUV5_t;

typedef struct
{                                   // bit      description
    uint16_t LPETHH:2;               // 1~0
    uint16_t LPETHL:2;               // 3~2
    uint16_t RSV1:2;                 // 5~4
    uint16_t BOOST_VEXT_MASK:1;      // 6
    uint16_t IZTHL:3;                // 9~7
    uint16_t RSVH:6;                 // 15~10
} DEV_OVUV6_bits;

typedef union  {
   uint16_t                 all;
   DEV_OVUV6_bits           bit;
} DEV_OVUV6_t;

typedef struct
{                                   // bit      description
    uint16_t DKI:3;                  // 2~0
    uint16_t SENCLK:1;               // 3
    uint16_t OHYS:2;                 // 5~4
    uint16_t DKP:3;                  // 8~6
    uint16_t MKP:2;                  // 10~9
    uint16_t RSVH:5;                 // 15~11
} DEV_TLOOP_CFG_bits;

typedef union  {
   uint16_t                 all;
   DEV_TLOOP_CFG_bits       bit;
} DEV_TLOOP_CFG_t;

typedef struct
{                                   // bit      description
    uint16_t GAINSIN:2;              // 1~0
    uint16_t GAINCOS:2;              // 2
    uint16_t RSVH:12;                // 15~4
} DEV_AFE_CFG_bits;

typedef union  {
   uint16_t                 all;
   DEV_AFE_CFG_bits         bit;
} DEV_AFE_CFG_t;

typedef struct
{                                   // bit      description
    uint16_t PHASEDEMOD:6;           // 5~0
    uint16_t EXTOUT:4;               // 9~6
    uint16_t EXTMODE:2;              // 11~10
    uint16_t APEN:1;                 // 12
    uint16_t PDEN:1;                 // 13
    uint16_t EXTUVF_CFG:2;           // 15~14
} DEV_PHASE_CFG_bits;

typedef union  {
   uint16_t                 all;
   DEV_PHASE_CFG_bits       bit;
} DEV_PHASE_CFG_t;

typedef struct
{                                   // bit      description
    uint16_t MODEVEXT:3;             // 2~0
    uint16_t SELFEXT:3;              // 5~3
    uint16_t RSV1:1;                 // 6
    uint16_t NPLE:2;                 // 8~7
    uint16_t RSV2:7;                 // 15~9
} DEV_CONFIG1_bits;

typedef union  {
   uint16_t                 all;
   DEV_CONFIG1_bits         bit;
} DEV_CONFIG1_t;

typedef struct
{                                   // bit      description
    uint16_t DIAGEXIT:1;             // 0
    uint16_t MEXTMON:1;              // 1
    uint16_t MAFECAL:1;              // 2
    uint16_t MIZUV:1;                // 3
    uint16_t MIZOV:1;                // 4
    uint16_t MEXTUV:1;               // 5
    uint16_t MEXTOV:1;               // 6
    uint16_t MFLOOPE:1;              // 7
    uint16_t MFOCOSOPL:1;            // 8
    uint16_t MFOSINOPL:1;            // 9
    uint16_t MFOCOSOPH:1;            // 10
    uint16_t MFOSINOPH:1;            // 11
    uint16_t RSV1:1;                 // 12
    uint16_t MFOSHORT:1;             // 13
    uint16_t RSV2:2;                 // 15-14
} DEV_CONTROL1_bits;

typedef union  {
   uint16_t                 all;
   DEV_CONTROL1_bits        bit;
} DEV_CONTROL1_t;

typedef struct
{                                   // bit      description
    uint16_t ENEXTUV:1;              // 0
    uint16_t ENEXTMON:1;             // 1
    uint16_t ENBISTF:1;              // 2
    uint16_t ENIOFAULT:1;            // 3
    uint16_t ENINFAULT:1;            // 4
    uint16_t RDC_DISABLE:1;          // 5
    uint16_t RSV1:8;                 // 13~6
    uint16_t LBIST_EN:1;             // 14
    uint16_t ABIST_EN:1;             // 15
} DEV_CONTROL2_bits;

typedef union  {
   uint16_t                 all;
   DEV_CONTROL2_bits        bit;
} DEV_CONTROL2_t;

typedef struct
{                                   // bit      description
    uint16_t EXTEN:1;                // 0
    uint16_t LPEN:1;                 // 1
    uint16_t ENSPIDIAG:1;            // 2
    uint16_t RSV1:13;                // 15~3
} DEV_CONTROL3_bits;

typedef union  {
   uint16_t                 all;
   DEV_CONTROL3_bits        bit;
} DEV_CONTROL3_t;

typedef struct
{                                   // bit      description
    uint16_t FOSHORT:1;              // 0
    uint16_t FGOPEN:1;               // 1
    uint16_t STAT:2;                 // 2
    uint16_t FOSINOPH:1;             // 4
    uint16_t FOCOSOPH:1;             // 5
    uint16_t FOSINOPL:1;             // 6
    uint16_t FOCOSOPL:1;             // 7
    uint16_t FLOOPE:1;               // 8
    uint16_t EXTOV:1;                // 9
    uint16_t EXTUV:1;                // 10
    uint16_t EXTILIM:1;              // 11
    uint16_t FTECRC:1;               // 12
    uint16_t FCECRC:1;               // 13
    uint16_t FRCRC:1;                // 14
    uint16_t FLOOP_CLAMP:1;          // 15
} DEV_STAT1_bits;

typedef union  {
   uint16_t                 all;
   DEV_STAT1_bits           bit;
} DEV_STAT1_t;

typedef struct
{                                   // bit      description
    uint16_t SORD:14;                // 13~0
    uint16_t SPRD:1;                 // 14
    uint16_t RSV1:1;                 // 15
} DEV_STAT2_bits;

typedef union  {
   uint16_t                 all;
   DEV_STAT2_bits           bit;
} DEV_STAT2_t;

typedef struct
{                                   // bit      description
    uint16_t FIZH1:1;                // 0
    uint16_t FIZH3:1;                // 1
    uint16_t FIZH2:1;                // 2
    uint16_t FIZH4:1;                // 3
    uint16_t FIZL1:1;                // 4
    uint16_t FIZL3:1;                // 5
    uint16_t FIZL2:1;                // 6
    uint16_t FIZL4:1;                // 7
    uint16_t OMIZ1H:1;               // 8
    uint16_t OMIZ3H:1;               // 9
    uint16_t OMIZ2H:1;               // 10
    uint16_t OMIZ4H:1;               // 11
    uint16_t OMIZ1L:1;               // 12
    uint16_t OMIZ3L:1;               // 13
    uint16_t OMIZ2L:1;               // 14
    uint16_t OMIZ4L:1;               // 15
} DEV_STAT3_bits;

typedef union  {
   uint16_t                 all;
   DEV_STAT3_bits           bit;
} DEV_STAT3_t;

typedef struct
{                                   // bit      description
    uint16_t SOUTZ:1;                // 0
    uint16_t SOUTB:1;                // 1
    uint16_t SOUTA:1;                // 2
    uint16_t SFAULT:1;               // 3
    uint16_t IOFAULT:1;              // 4
    uint16_t FVDDOV:1;               // 5
    uint16_t FVCCOV:1;               // 6
    uint16_t LBISTF:1;               // 7
    uint16_t ABISTF:1;               // 8
    uint16_t FEXTMODE:1;             // 9
    uint16_t FTSD2:1;                // 10
    uint16_t FVDDOC:1;               // 11
    uint16_t FBSTOV:1;               // 12
    uint16_t SPI_ERR:1;              // 13
    uint16_t FEXTMONL:1;             // 14
    uint16_t FEXTMONH:1;             // 15
} DEV_STAT4_bits;

typedef union  {
   uint16_t                 all;
   DEV_STAT4_bits           bit;
} DEV_STAT4_t;

typedef struct
{                                   // bit      description
    uint16_t ORDANGLE:13;            // 12~0
    uint16_t ORDCLOCK:1;             // 13
    uint16_t PRD:1;                  // 14
    uint16_t RSV1:1;                 // 15
} DEV_STAT5_bits;

typedef union  {
   uint16_t                 all;
   DEV_STAT5_bits           bit;
} DEV_STAT5_t;

typedef struct
{                                   // bit      description
    uint16_t ORDVELOCITY:14;         // 13~0
    uint16_t PRD:1;                  // 14
    uint16_t RSV1:1;                 // 15
} DEV_STAT6_bits;

typedef union  {
   uint16_t                 all;
   DEV_STAT6_bits           bit;
} DEV_STAT6_t;

typedef struct
{                                   // bit      description
    uint16_t REVID:3;                // 2~0
    uint16_t OPTID:3;                // 5~3
    uint16_t DEVSTATE:1;             // 6
    uint16_t FAFECAL:1;              // 7
    uint16_t RSV1:8;                 // 15~8
} DEV_STAT7_bits;

typedef union  {
   uint16_t                 all;
   DEV_STAT7_bits           bit;
} DEV_STAT7_t;

typedef struct
{                                   // bit      description
    uint16_t ECCRC:8;                // 7~0
    uint16_t RSV1:8;                 // 15~8
} DEV_CLCRC_bits;

typedef union  {
   uint16_t                 all;
   DEV_CLCRC_bits           bit;
} DEV_CLCRC_t;

typedef struct
{                                   // bit      description
    uint16_t RCRC:8;                 // 7~0
    uint16_t RSV1:8;                 // 15~8
} DEV_CRC_bits;

typedef union  {
   uint16_t                 all;
   DEV_CRC_bits             bit;
} DEV_CRC_t;

typedef struct
{                                   // bit      description
    uint16_t CRCRC:8;                // 7~0
    uint16_t RSV1:8;                 // 15~8
} CRCCALC_bits;

typedef union  {
   uint16_t                 all;
   CRCCALC_bits             bit;
} CRCCALC_t;

typedef struct
{                                   // bit      description
    uint16_t EECMD:8;                // 7~0
    uint16_t RSV1:8;                 // 15~8
} DEV_EE_CTRL1_bits;

typedef union  {
   uint16_t                 all;
   DEV_EE_CTRL1_bits        bit;
} DEV_EE_CTRL1_t;

typedef struct
{                                   // bit      description
    uint16_t CRCCTL:1;               // 0
    uint16_t RSV1:15;                // 15~1
} DEV_CRC_CTRL1_bits;

typedef union  {
   uint16_t                 all;
   DEV_CRC_CTRL1_bits       bit;
} DEV_CRC_CTRL1_t;

typedef struct
{                                   // bit      description
    uint16_t EEUNLK:8;               // 7~0
    uint16_t RSV1:8;                 // 15~8
} DEV_EE_CTRL4_bits;

typedef union  {
   uint16_t                 all;
   DEV_EE_CTRL4_bits        bit;
} DEV_EE_CTRL4_t;

typedef struct
{                                   // bit      description
    uint16_t DEVUNLK:8;              // 7~0
    uint16_t RSV1:8;                 // 15~8
} DEV_UNLK_CTRL1_bits;

typedef union  {
   uint16_t                 all;
   DEV_UNLK_CTRL1_bits      bit;
} DEV_UNLK_CTRL1_t;

typedef struct  _PGA411Q1_Vars_t_
{
    DEV_OVUV1_t             DEV_OVUV1;
    DEV_OVUV2_t             DEV_OVUV2;
    DEV_OVUV3_t             DEV_OVUV3;
    DEV_OVUV4_t             DEV_OVUV4;
    DEV_OVUV5_t             DEV_OVUV5;
    DEV_TLOOP_CFG_t         DEV_TLOOP_CFG;
    DEV_AFE_CFG_t           DEV_AFE_CFG;
    DEV_PHASE_CFG_t         DEV_PHASE_CFG;
    DEV_CONFIG1_t           DEV_CONFIG1;
    DEV_CONTROL1_t          DEV_CONTROL1;
    DEV_CONTROL2_t          DEV_CONTROL2;
    DEV_CONTROL3_t          DEV_CONTROL3;
    DEV_STAT1_t             DEV_STAT1;
    DEV_STAT2_t             DEV_STAT2;
    DEV_STAT3_t             DEV_STAT3;
    DEV_STAT4_t             DEV_STAT4;
    DEV_STAT5_t             DEV_STAT5;
    DEV_STAT6_t             DEV_STAT6;
    DEV_STAT7_t             DEV_STAT7;
    DEV_CLCRC_t             DEV_CLCRC;
    DEV_CRC_t               DEV_CRC;
    CRCCALC_t               CRCCALC;
    DEV_EE_CTRL1_t          DEV_EE_CTRL1;
    DEV_CRC_CTRL1_t         DEV_CRC_CTRL1;
    DEV_EE_CTRL4_t          DEV_EE_CTRL4;
    DEV_UNLK_CTRL1_t        DEV_UNLK_CTRL1;
    bool                    readStat;           //!<
    bool                    readCmd;            //!<
    bool                    writeCmd;           //!<
    bool                    readAngle;          //!<
    bool                    readVelocity;       //!<

    _iq angleOffset;            //!< IQ math angle
    _iq angle;                  //!< IQ math angle
    _iq angle_pu;               //!< IQ math angle
    _iq velocity_pu;            //!< IQ math velocity
    _iq velocityCoef;           //!< IQ math velocity coeff
} PGA411Q1_Vars_t;

extern PGA411Q1_Vars_t gPga411q1Vars;

/******************************************************************************
 * VARIABLES, CONSTANTS
 *****************************************************************************/

/******************************************************************************
 * SYSTEM INTERFACE FUNCTIONS PROTOTYPES
 *****************************************************************************/

/* Initialization PGA411-Q1 related hardware */
void pga411_Init(void);

/* Perform PGA411-Q1 reset using GPIO */
void pga411_Reset(void);

/* Perform PGA411-Q1 FAULT reset using GPIO */
void pga411_FaultReset(void);

/* Read data from defined register */
uint16_t pga411_ReadReg(uint16_t reg);

/* Write data to defined register */
void pga411_WriteReg(uint16_t reg, uint16_t data);

/* Read all registers from PGA */
void pga411_ReadAll(void);

/* Change state diagnostic/normal */
void pga411_State(uint16_t state);

/* Device unlock (must be in diagnostic state) */
void pga411_DeviceUnlock(void);

/* EEPROM unlock (must be in diagnostic state) */
void pga411_EEPROMUnlock(void);

/* Write default configuration to PGA */
void pga411_DefaultCfg(PGA411Q1_Vars_t *pga411q1);

void pga411_GetAngle(PGA411Q1_Vars_t *pga411q1);

void pga411_GetVelocity(PGA411Q1_Vars_t *pga411q1);

#endif
