/******************************************************************************
* TIDA-01507 Resolver-to-Uart functionality only (based on TIDA-00796)
* Module: Hardware Abstraction Layer
* Author: Jiri Panacek, j-panacek@ti.com
*
* Copyright 2016 Texas Instruments Incorporated. All rights reserved.
******************************************************************************/

#include "sw/safety/pga411q1/src/32b/f28x/f2806x/pga411q1_spi.h"

/******************************************************************************
 * MACROS
 *****************************************************************************/

#define    EMPTY_LOOP
#define    ENDLESS    1

/* PUSH data to SPI buffer - left adjusted */
#define SPIA_PUSH(TX) SpiaRegs.SPITXBUF = (TX << 8)

/* POP data from SPI buffer - right adjusted */
#define SPIA_POP SpiaRegs.SPIRXBUF


/******************************************************************************
 * ENUMS
 *****************************************************************************/

/******************************************************************************
 * TYPEDEFS
 *****************************************************************************/

/******************************************************************************
 * VARIABLES, CONSTANTS
 *****************************************************************************/

/******************************************************************************
 * INTERNAL FUNCTIONS PROTOTYPES
 *****************************************************************************/

/* PGA411-Q1 GPIO configuration */
void pga411_InitGPIO(void)
{
    /* note: for better portability we configure all pins again */
    asm(" EALLOW");                         /* Enable EALLOW protected register access */

    /* FAULTRES, negative logic */
    GpioDataRegs.GPBSET.bit.GPIO51 = 1;     /* default value 1, ignored anyway */

    /* BMODE0 - low = 10b resolution, high = 12b resolution */
    GpioDataRegs.GPASET.bit.GPIO28 = 1;     /* default 12b resolution */

    /* AMODE - accelerated mode */
    GpioDataRegs.GPACLEAR.bit.GPIO8 = 1;    /* accelerated mode disabled by default */

    /* FAULT - fault detection, active = high - Z */
    GpioDataRegs.GPBSET.bit.GPIO51 = 1;    /* default value 1, ignored anyway */

    /* OMODE - output mode */
    GpioDataRegs.GPBCLEAR.bit.GPIO52 = 1;    /*  */
    /*
     * VA0 | VA1 | ORD [11:0] Function
     * ----+-----+------------------------------
     *  0  |  0  |  Hi-Z
     *  1  |  1  |  Hi-Z
     *  0  |  1  |  Angle Output at ORD [11:0]
     *  1  |  0  |  Velocity output at ORD [11:0]
     */
    /* VA0 - ORD[11:0] Output Selection */
    GpioDataRegs.GPACLEAR.bit.GPIO6 = 1;  /* VA0 = 0 */

    /* VA1 - ORD[11:0] Output Selection */
    GpioDataRegs.GPASET.bit.GPIO7 = 1;    /* VA1=1 */

    asm(" EDIS"); /* Disable EALLOW protected register access */
}

/* SPI init */
void pga411_InitSPI(void)
{
    asm(" EALLOW"); /* Enable EALLOW protected register access */
    /* enable peripheral clock */
    SysCtrlRegs.PCLKCR0.bit.SPIAENCLK = 1; /* SPI-A */

    /* Initialize SPI periphery */
    SpiaRegs.SPICCR.bit.SPISWRESET = 0; /* Reset SPI and hold */
    SpiaRegs.SPICCR.all = 0x0007;       /* Loopback off  8-bit character, Clock Polarity 0 */
    SpiaRegs.SPICTL.all = 0x0006;       /* master, xmit enable */
    SpiaRegs.SPIBRR = 14;               /* SYSCLK = 90Mhz = LSPCLK, baud rate = LSPCLK/(SPIBRR+1) = 6 MHz */
    SpiaRegs.SPISTS.all = 0x0000;       /* reset all status information */
    SpiaRegs.SPIPRI.bit.FREE = 1;       /* priority control settings - free run */
    SpiaRegs.SPICCR.bit.SPISWRESET = 1; /* Release from SPI reset */
    /* reset both fifos */
    SpiaRegs.SPIFFTX.bit.TXFIFO = 0;
    SpiaRegs.SPIFFRX.bit.RXFIFORESET = 0; /*and keep them in reset during configuration */
    /* configure fifos  */
    SpiaRegs.SPIFFTX.all = 0xC004; /* Enable FIFO's, do not use interrupts */
    SpiaRegs.SPIFFRX.all = 0x0004; /* Set RX FIFO, do not use interrupts  */
    SpiaRegs.SPIFFCT.all = 0x0000; /* no delay between FIFO and TXBUF */
    /* releasing fifos from reset */
    SpiaRegs.SPIFFTX.bit.TXFIFO = 1;
    SpiaRegs.SPIFFRX.bit.RXFIFORESET = 1; /* OK */

    /* error checking, fifo pointers should be zero */
    if ((SpiaRegs.SPIFFTX.bit.TXFFST != 0) || (SpiaRegs.SPIFFRX.bit.RXFFST != 0)) { pga411_assert(); }

    asm(" EDIS"); /* Disable EALLOW protected register access */
}

/* Exchanges 4 Bytes via SPI, blocks until finished, 32b architecture = no pointer */
uint32_t pga411_Xmit4BSPI(uint32_t frame)
{
    uint32_t rxval = 0x00000000; /* incoming data */

    GpioDataRegs.GPACLEAR.bit.GPIO19 = 1;;      // Enable SCS

    /* empty RX and TX 4B FIFOs */
    SpiaRegs.SPIFFTX.bit.TXFIFO = 0; /* reset FIFOs */
    SpiaRegs.SPIFFRX.bit.RXFIFORESET = 0;
    asm(" NOP"); /* probably not needed */
    asm(" NOP"); /* probably not needed */
    SpiaRegs.SPIFFTX.bit.TXFIFO = 1;
    SpiaRegs.SPIFFRX.bit.RXFIFORESET = 1; /* release from reset */

    /* quick & dirty ! */
    SPIA_PUSH(frame >> 24); /* first byte (from most significant) */
    SPIA_PUSH(frame >> 16);
    SPIA_PUSH(frame >> 8);
    SPIA_PUSH(frame); /* last byte */

    /* wait until the transmission is finished */
    while (SpiaRegs.SPIFFRX.bit.RXFFST < 4) {EMPTY_LOOP};

    /* get data from RX FIFO */
    rxval = SPIA_POP; /* first byte (from the most significant) */

    rxval = ((rxval << 8) | SPIA_POP);

    rxval = ((rxval << 8) | SPIA_POP);

    rxval = ((rxval << 8) | SPIA_POP); /* last byte */

    /* NCS must remain high at least 200ns */
    usDelay(1);

    GpioDataRegs.GPASET.bit.GPIO19 = 1;         // Disable SCS

    return (rxval);
}

/* pseudo assert */
void pga411_assert(void)
{
    __asm("        ESTOP0"); /* stops emulation at this point */
}

