/*
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * @file   tps65381.c
 * @author Christian Herget
 * @date   2/3/2017
 * @brief  TPS65381 driver, provides functions to access registers via SPI
 *
 */
#include "F2806x_Device.h"     // F2806x Headerfile Include File

#include "sw/modules/types/src/types.h"
#include "sw/modules/usDelay/src/32b/usDelay.h"

#include "sw/safety/tps65381a/src/32b/f28x/f2806x/tps65381a_spi.h"
#include "sw/safety/tps65381a/src/32b/f28x/f2806x/tps65381a.h"

TPS65381A_Vars_t gTps65381aVars;

void TPS65381A_InitRegister(TPS65381A_Vars_t *tps65381a)
{
    tps65381a->wdTrigState = WD_TRIG_STANDBY;

    tps65381a->wdTrigTimeCnt = 0;               //!<
    tps65381a->wdTrigTimeSet = 700;             //!<

    tps65381a->readCtrlCmd = false;             //!<
    tps65381a->readStatCmd = false;
    tps65381a->writeCtrlCmd = false;

    tps65381a->writeWdCfgCmd = false;
    tps65381a->ReadWdStatCmd = false;

    return;
}


void TPS65381A_Init(TPS65381A_Vars_t *tps65381a)
{
    TPS65381A_InitSPI();
    gSpiMode = SPI_TPS65381;

    // Unlock listed registers
    TPS65381A_SPI_Write(SW_UNLOCK, 0xAA);
    TPS65381A_SPI_Write(WR_SAFETY_ERR_CFG, 0x20);

    tps65381a->DEV_ID = TPS65381A_SPI_Read(RD_DEV_ID)&0x00FF;
    tps65381a->DEV_REV = TPS65381A_SPI_Read(RD_DEV_REV)&0x00FF;

    tps65381a->SAFETY_STAT_5.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_5)&0x00FF;
    tps65381a->SAFETY_CHECK_CTRL.all = TPS65381A_SPI_Read(RD_SAFETY_CHECK_CTRL)&0x00FF;

    tps65381a->wdTrigTimeCnt = 0;

    tps65381a->SAFETY_STAT_2.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_2)&0x00FF;
    tps65381a->SAFETY_STAT_4.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_4)&0x00FF;

    while((tps65381a->SAFETY_STAT_5.bit.STAT_FSM != FSM_DIAGNOSTIC) && (tps65381a->SAFETY_STAT_5.bit.STAT_FSM != FSM_ACTIVE))
    {
        tps65381a->SAFETY_CHECK_CTRL.bit.DIAG_EXIT = 0;
        tps65381a->SAFETY_CHECK_CTRL.bit.DIAG_EXIT_MASK = 1;
        TPS65381A_SPI_Write(WR_SAFETY_CHECK_CTRL, tps65381a->SAFETY_CHECK_CTRL.all);

        tps65381a->SAFETY_STAT_5.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_5)&0x00FF;
        tps65381a->SAFETY_CHECK_CTRL.all = TPS65381A_SPI_Read(RD_SAFETY_CHECK_CTRL)&0x00FF;
        usDelay(100);
    }

    if(tps65381a->SAFETY_STAT_5.bit.STAT_FSM == FSM_DIAGNOSTIC)
    {
        // Staty in DigState
        tps65381a->SAFETY_CHECK_CTRL.all = TPS65381A_SPI_Read(RD_SAFETY_CHECK_CTRL)&0x00FF;

        // Ensure that ENDRV is disabled
        tps65381a->SAFETY_CHECK_CTRL.bit.DIAG_EXIT = 0;
        tps65381a->SAFETY_CHECK_CTRL.bit.DIAG_EXIT_MASK = 1;
        tps65381a->SAFETY_CHECK_CTRL.bit.ENABLE_DRV = 0;
        TPS65381A_SPI_Write(WR_SAFETY_CHECK_CTRL, tps65381a->SAFETY_CHECK_CTRL.all);

//        // Close Window(WIN1)=20.3~22.5ms, Open Window(WIN2)=8.9ms~9.8ms
//        TPS65381A_SPI_Write(WR_WDT_WIN1_CFG, 0x28);
//        TPS65381A_SPI_Write(WR_WDT_WIN2_CFG, 0x10);

        // Close Window(WIN1)=37.1~41.0ms, Open Window(WIN2)=13.1ms~14.3ms
        TPS65381A_SPI_Write(WR_WDT_WIN1_CFG, 0x48);
        TPS65381A_SPI_Write(WR_WDT_WIN2_CFG, 0x18);

        tps65381a->wdTrigTimeCnt = 0;
    }

    tps65381a->SAFETY_FUNC_CFG.all = TPS65381A_SPI_Read(RD_SAFETY_FUNC_CFG)&0x00FF;
    tps65381a->SAFETY_FUNC_CFG.bit.WD_RST_EN = 0;
    tps65381a->SAFETY_FUNC_CFG.bit.WD_CFG = 0;          // Trigger mode

    // Clear error status registers
    TPS65381A_SPI_Write(WR_SAFETY_ERR_STAT, 0x00);
    tps65381a->SAFETY_ERR_STAT.all = TPS65381A_SPI_Read(RD_SAFETY_ERR_STAT)&0x00FF;

    // Set ACTIVE State
    tps65381a->SAFETY_CHECK_CTRL.bit.DIAG_EXIT = 1;
    tps65381a->SAFETY_CHECK_CTRL.bit.DIAG_EXIT_MASK = 0;
    TPS65381A_SPI_Write(WR_SAFETY_CHECK_CTRL, tps65381a->SAFETY_CHECK_CTRL.all);

    tps65381a->SAFETY_STAT_2.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_2)&0x00FF;

    while(tps65381a->SAFETY_STAT_2.bit.WD_FAIL_CNT > 4)
    {
        tps65381a->SAFETY_STAT_2.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_2)&0x00FF;
        tps65381a->SAFETY_STAT_5.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_5)&0x00FF;
    }

    // Set EN_DRV
    tps65381a->SAFETY_CHECK_CTRL.bit.ENABLE_DRV = 1;
    TPS65381A_SPI_Write(WR_SAFETY_CHECK_CTRL, tps65381a->SAFETY_CHECK_CTRL.all);

    tps65381a->SAFETY_STAT_4.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_4)&0x00FF;

    return;
}

void TPS65381A_ReadWdStatus(TPS65381A_Vars_t *tps65381a)
{
    if(tps65381a->ReadWdStatCmd == true)
    {
        if(gSpiMode != SPI_TPS65381)
        {
            TPS65381A_InitSPI();
            gSpiMode = SPI_TPS65381;
        }

        tps65381a->WD_STATUS.all = TPS65381A_SPI_Read(RD_WDT_STATUS)&0x00FF;
        tps65381a->DEV_STAT.all = TPS65381A_SPI_Read(RD_DEV_STAT)&0x00FF;

        tps65381a->ReadWdStatCmd = false;
    }
}

void TPS65381A_ReadSafetyStatus(TPS65381A_Vars_t *tps65381a)
{
    if(tps65381a->readStatCmd == true)
    {
        if(gSpiMode != SPI_TPS65381)
        {
            TPS65381A_InitSPI();
            gSpiMode = SPI_TPS65381;
        }

        tps65381a->SAFETY_STAT_1.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_1)&0x00FF;
        tps65381a->SAFETY_STAT_2.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_2)&0x00FF;
        tps65381a->SAFETY_STAT_3.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_3)&0x00FF;
        tps65381a->SAFETY_STAT_4.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_4)&0x00FF;
        tps65381a->SAFETY_STAT_5.all = TPS65381A_SPI_Read(RD_SAFETY_STAT_5)&0x00FF;

        tps65381a->DEV_STAT.all = TPS65381A_SPI_Read(RD_DEV_STAT)&0x00FF;
        tps65381a->VMON_STAT_1.all = TPS65381A_SPI_Read(RD_VMON_STAT_1)&0x00FF;
        tps65381a->VMON_STAT_2.all = TPS65381A_SPI_Read(RD_VMON_STAT_2)&0x00FF;

        tps65381a->readStatCmd = false;
    }
}



