/*
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * @file   tps65381.h
 * @author Christian Herget
 * @date   2/3/2017
 * @brief  Header file for TPS65381 driver
 *
 */

#ifndef TPS65381_H_
#define TPS65381_H_

#include <stdint.h>

#include "F2806x_Device.h"     // F2806x Headerfile Include File

#include "sw/safety/com_spi/src/32b/f28x/com_spi.h"
#include "sw/modules/types/src/types.h"
#include "sw/modules/usDelay/src/32b/usDelay.h"
#include "sw/modules/iqmath/src/32b/IQmathLib.h"

/* Register Read Definitions */
#define RD_DEV_ID             (0x06u)
#define RD_DEV_REV            (0x0Cu)
#define RD_DEV_CFG1           (0xAFu)
#define RD_DEV_CFG2           (0x48u)
#define RD_SAFETY_STAT_1      (0x24u)
#define RD_SAFETY_STAT_2      (0xC5u)
#define RD_SAFETY_STAT_3      (0xA3u)
#define RD_SAFETY_STAT_4      (0xA5u)
#define RD_SAFETY_STAT_5      (0xC0u)
#define RD_SAFETY_ERR_CFG     (0x30u)
#define RD_SAFETY_ERR_STAT    (0xAAu)
#define RD_SAFETY_PWD_THR_CFG (0x39u)
#define RD_SAFETY_CHECK_CTRL  (0x44u)
#define RD_SAFETY_BIST_CTRL   (0x3Cu)
#define RD_WDT_WIN1_CFG       (0x2Eu)
#define RD_WDT_WIN2_CFG       (0x05u)
#define RD_WD_OPEN_WIN_CFG    (RD_WDT_WIN1_CFG)
#define RD_WD_CLOSE_WIN_CFG   (RD_WDT_WIN2_CFG)
#define RD_WDT_TOKEN_VALUE    (0x36u)
#define RD_WDT_STATUS         (0x4Eu)
#define RD_DEV_STAT           (0x11u)
#define RD_VMON_STAT_1        (0x12u)
#define RD_VMON_STAT_2        (0xA6u)
#define RD_SENS_CTRL          (0x56u)
#define RD_SAFETY_FUNC_CFG    (0x3Au)
#define RD_SAFE_CFG_CRC       (0x5Au)
#define RD_DIAG_CFG_CTRL      (0xDDu)
#define RD_DIAG_MUX_SEL       (0xACu)
#define RD_SAFETY_ERR_PWM_H   (0xD7u)
#define RD_SAFETY_ERR_PWM_L   (0x59u)
#define RD_WDT_TOKEN_FDBCK    (0x78u)

/* Register Write Definitions */
#define SW_LOCK               (0xBDU)
#define SW_UNLOCK             (0xBBu)
#define WR_DEV_CFG1           (0xB7u)
#define WR_DEV_CFG2           (0x95u)
#define WR_CAN_STBY           (0x7Du)
#define WR_SAFETY_ERR_CFG     (0xDBu)
#define WR_SAFETY_ERR_STAT    (0xA9u)
#define WR_SAFETY_PWD_THR_CFG (0x99u)
#define WR_SAFETY_CHECK_CTRL  (0x93u)
#define WR_SAFETY_BIST_CTRL   (0x9Fu)
#define WR_WDT_WIN1_CFG       (0xEDu)
#define WR_WDT_WIN2_CFG       (0x09u)
#define WR_OPEN_WD            (WR_WDT_WIN1_CFG)
#define WR_CLOSE_WD           (WR_WDT_WIN2_CFG)
#define WR_WDT_ANSWER         (0xE1u)
#define WR_SENS_CTRL          (0x7Bu)
#define WR_SAFETY_FUNC_CFG    (0x35u)
#define WR_SAFE_CFG_CRC       (0x63u)
#define WR_DIAG_CFG_CTRL      (0xCCu)
#define WR_DIAG_MUX_SEL       (0xC9u)
#define WR_SAFETY_ERR_PWM_H   (0xD8u)
#define WR_SAFETY_ERR_PWM_L   (0x7Eu)
#define WR_WDT_TOKEN_FDBCK    (0x77u)

typedef enum
{
    WD_TRIG_STANDBY   = 0x00,
    WD_TRIG_CMD       = 0x01,
    WD_TRIG_SET       = 0x02
} WDState_e;

typedef enum
{
    TPS_STANDBY_STATE    = 0x00,
    TPS_RESET_STATE      = 0x03,
    TPS_DIAGNOSTIC_STATE = 0x07,
    TPS_ACTIVE_STATE     = 0x05,
    TPS_SAFE_STATE       = 0x04
} TPSState_e;

typedef struct
{                                   // bit      description
    uint16_t Data:8;                // 0~7
    uint16_t Addr:8;                // 8~15
}TPS65381A_Data_t;

typedef struct
{
    TPS65381A_Data_t    AddrData;
    // and finaly the whole frame
    uint16_t            frame;
} TPS65381A_spi_frame_t;

typedef struct
{                                   // bit      description
    uint16_t IGN:1;                 // 0 Deglitched IGN pi
    uint16_t CANWU_L:1;             // 1 Latched CAN wake-up event
    uint16_t RSV1:6;                // 7~3
    uint16_t RSVH:8;                // 15~8
}DEV_STAT_bits;

typedef union  {
   uint16_t         all;
   DEV_STAT_bits    bit;
} DEV_STAT_t;

typedef enum
{
    VDD_5V_SEL = 0,                  //!< 5-V setting
    VDD_3p3V_SEL = 1                 //!< 3.3-V setting
} VDD_3_5_SEL_e;

typedef struct
{                                   // bit      description
    uint16_t RSV1:6;                // 0~5 RSV
    uint16_t NMASK_VDD1_UV_OV:1;    // 6
    VDD_3_5_SEL_e VDD_3_5_SEL:1;    // 7
    uint16_t RSVH:8;                // 15~8
} DEV_CFG1_bits;

typedef union  {
   uint16_t         all;
   DEV_CFG1_bits    bit;
} DEV_CFG1_t;

typedef struct
{                                   // bit      description
    uint16_t RSV1:4;                // 0~3 RSV
    uint16_t POST_RUN_RST:1;        // 4
    uint16_t MASK_VBATP_OV:1;       // 5
    uint16_t NMASK_VDD5_OT:1;       // 6
    uint16_t NMASK_VDD3o5_OT:1;     // 7
    uint16_t RSVH:8;                // 15~8
} DEV_CFG2_bits;

typedef union  {
   uint16_t         all;
   DEV_CFG2_bits    bit;
} DEV_CFG2_t;

typedef struct
{                                   // bit      description
    uint16_t BG_ERR1:1;             // 0
    uint16_t BG_ERR2:1;             // 1
    uint16_t AVDD_VMON_ERR:1;       // 2
    uint16_t VCP12_UV:1;            // 3
    uint16_t VCP12_OV:1;            // 4
    uint16_t VCP17_OV:1;            // 5
    uint16_t VCP17_UV:1;            // 6
    uint16_t VBATP_OV:1;            // 7
    uint16_t RSVH:8;                // 15~8
} VMON_STAT_1_bits;

typedef union  {
   uint16_t         all;
   VMON_STAT_1_bits bit;
} VMON_STAT_1_t;

typedef struct
{                                   // bit      description
    uint16_t VDD1_UV:1;             // 0
    uint16_t VDD1_OV:1;             // 1
    uint16_t VDD3o5_UV:1;           // 2
    uint16_t VDD3o5_OV:1;           // 3
    uint16_t VDD5_UV:1;             // 4
    uint16_t VDD5_OV:1;             // 5
    uint16_t VDD6_UV:1;             // 6
    uint16_t VDD6_OV:1;             // 7
    uint16_t RSVH:8;                // 15~8
} VMON_STAT_2_bits;

typedef union  {
   uint16_t         all;
   VMON_STAT_2_bits bit;
} VMON_STAT_2_t;

typedef struct
{                                   // bit      description
    uint16_t VDD_3_5_OT:1;          // 0
    uint16_t VDD5_OT:1;             // 1
    uint16_t VSOUT1_OT:1;           // 2
    uint16_t RSV:1;                 // 3
    uint16_t VSOUT1_OV:1;           // 4
    uint16_t VSOUT1_UV:1;           // 5
    uint16_t VDD3o5_ILIM:1;         // 6
    uint16_t VDD5_ILIM:1;           // 7
    uint16_t RSVH:8;                // 15~8
} SAFETY_STAT_1_bits;

typedef union  {
   uint16_t             all;
   SAFETY_STAT_1_bits   bit;
} SAFETY_STAT_1_t;

typedef struct
{                                   // bit      description
    uint16_t WD_FAIL_CNT:3;         // 2~0
    uint16_t RSV1:1;                // 3
    uint16_t EE_CRC_ERR:1;          // 4
    uint16_t CFG_CRC_ERR:1;         // 5
    uint16_t RSV2:2;                // 7~6
    uint16_t RSVH:8;                // 15~8
} SAFETY_STAT_2_bits;

typedef union  {
   uint16_t             all;
   SAFETY_STAT_2_bits   bit;
} SAFETY_STAT_2_t;

typedef struct
{                                   // bit      description
    uint16_t ABIST_RUN:1;           // 0
    uint16_t LBIST_RUN:1;           // 1
    uint16_t ABIST_ERR:1;           // 2
    uint16_t ABIST_ERR2:1;          // 3
    uint16_t LBIST_ERR:1;           // 4
    uint16_t NRES_ERR:1;            // 5
    uint16_t RSV1:2;                // 7~6
    uint16_t RSVH:8;                // 15~8
} SAFETY_STAT_3_bits;

typedef union  {
   uint16_t             all;
   SAFETY_STAT_3_bits   bit;
} SAFETY_STAT_3_t;

typedef struct
{                                   // bit      description
    uint16_t TRIM_ERR_VMON:1;       // 0
    uint16_t ENDRV_ERR:1;           // 1
    uint16_t WD_ERR:1;              // 2
    uint16_t MCU_ERR:1;             // 3
    uint16_t RSV1:1;                // 4
    uint16_t LOCLK:1;               // 5
    uint16_t SPI_ERR:2;             // 7~6
    uint16_t RSVH:8;                // 15~8
} SAFETY_STAT_4_bits;

typedef union  {
   uint16_t             all;
   SAFETY_STAT_4_bits   bit;
} SAFETY_STAT_4_t;

typedef enum
{
    FSM_STANDBY = 0,                //!<
    FSM_RESET = 3,                  //!<
    FSM_DIAGNOSTIC = 7,             //!<
    FSM_ACTIVE = 5,                 //!<
    FSM_SAFE = 4                    //!<
} STAT_FSM_e;

typedef struct
{                                   // bit      description
    STAT_FSM_e STAT_FSM:3;          // 2~0
    uint16_t RSV1:5;                // 7~3 RSV
    uint16_t RSVH:8;                // 15~8
} SAFETY_STAT_5_bits;

typedef union  {
   uint16_t             all;
   SAFETY_STAT_5_bits   bit;
} SAFETY_STAT_5_t;

typedef struct
{                                   // bit      description
    uint16_t CFG_LOCK:1;            // 0
    uint16_t SAFE_LOCK_THR:4;       // 4~1
    uint16_t SAFE_TO:3;             // 7~5
    uint16_t RSVH:8;                // 15~8
} SAFETY_ERR_CFG_bits;

typedef union  {
   uint16_t             all;
   SAFETY_ERR_CFG_bits  bit;
} SAFETY_ERR_CFG_t;

typedef struct
{                                   // bit      description
    uint16_t ABIST_EN:1;            // 0
    uint16_t ABIST_EN2:1;           // 1
    uint16_t LBIST_EN:1;            // 2
    uint16_t RSV1:1;                // 3
    uint16_t EE_CRC_CHK:1;          // 4
    uint16_t AUTO_BIST_DIS:1;       // 5
    uint16_t BIST_DEG_CN:2;         // 7~6
    uint16_t RSVH:8;                // 15~8
} SAFETY_BIST_CTRL_bits;

typedef union  {
   uint16_t                 all;
   SAFETY_BIST_CTRL_bits    bit;
} SAFETY_BIST_CTRL_t;

typedef struct
{                                   // bit      description
    uint16_t DIAG_EXIT:1;           // 0
    uint16_t DIAG_EXIT_MASK:1;      // 1
    uint16_t NO_ERROR:1;            // 2
    uint16_t RSV1:2;                // 4~3
    uint16_t ENABLE_DRV:1;          // 5
    uint16_t RSV2:1;                // 6
    uint16_t CFG_CRC_EN:1;          // 7
    uint16_t RSVH:8;                // 15~8
} SAFETY_CHECK_CTRL_bits;

typedef union  {
   uint16_t                 all;
   SAFETY_CHECK_CTRL_bits   bit;
} SAFETY_CHECK_CTRL_t;

typedef struct
{                                   // bit      description
    uint16_t VDD_3_5_SEL:1;         // 0
    uint16_t RSV:1;                 // 1
    uint16_t DIS_NRES_MON:1;        // 2
    uint16_t WD_RST_EN:1;           // 3
    uint16_t IGN_PWRL:1;            // 4
    uint16_t WD_CFG:1;              // 5
    uint16_t ERROR_CFG:1;           // 6
    uint16_t NO_SAFE_TO:1;          // 7
    uint16_t RSVH:8;                // 15~8
} SAFETY_FUNC_CFG_bits;

typedef union  {
   uint16_t                 all;
   SAFETY_FUNC_CFG_bits     bit;
} SAFETY_FUNC_CFG_t;

typedef struct
{                                   // bit      description
    uint16_t DEV_ERR_CNT:4;         // 3~0
    uint16_t WD_FAIL:1;             // 4
    uint16_t ERROR_PIN_FAIL:1;      // 5
    uint16_t RSV1:1;                // 7~6
    uint16_t RSVH:8;                // 15~8
} SAFETY_ERR_STAT_bits;

typedef union  {
   uint16_t                 all;
   SAFETY_ERR_STAT_bits     bit;
} SAFETY_ERR_STAT_t;

typedef struct
{                                   // bit      description
    uint16_t ERR_PWMH:8;            // 7~0
    uint16_t RSVH:8;                // 15~8
} SAFETY_ERR_PWM_H_bits;

typedef union  {
   uint16_t                 all;
   SAFETY_ERR_PWM_H_bits    bit;
} SAFETY_ERR_PWM_H_t;

typedef struct
{                                   // bit      description
    uint16_t ERR_PWML:8;            // 7~0
    uint16_t RSVH:8;                // 15~8
} SAFETY_ERR_PWM_L_bits;

typedef union  {
   uint16_t                 all;
   SAFETY_ERR_PWM_L_bits    bit;
} SAFETY_ERR_PWM_L_t;

typedef struct
{                                   // bit      description
    uint16_t PWD_THR:4;             // 3~0
    uint16_t RSV:4;                 // 7~4
    uint16_t RSVH:8;                // 15~8
} SAFETY_PWD_THR_CFG_bits;

typedef union  {
   uint16_t                 all;
   SAFETY_PWD_THR_CFG_bits  bit;
} SAFETY_PWD_THR_CFG_t;

typedef struct
{                                   // bit      description
    uint16_t CFG_CRC:8;             // 7~0
    uint16_t RSVH:8;                // 15~8
} SAFETY_CFG_CRC_bits;

typedef union  {
   uint16_t                 all;
   SAFETY_CFG_CRC_bits      bit;
} SAFETY_CFG_CRC_t;

typedef struct
{                                   // bit      description
    uint16_t MUX_CFG:2;             // 1~0
    uint16_t INT_CON:3;             // 4~2
    uint16_t MUX_OUT:1;             // 5
    uint16_t SPI_SDO:1;             // 6
    uint16_t MUX_EN:1;              // 7
    uint16_t RSVH:8;                // 15~8
} DIAG_CFG_CTRL_bits;

typedef union  {
   uint16_t                 all;
   DIAG_CFG_CTRL_bits       bit;
} DIAG_CFG_CTRL_t;

typedef struct
{                                   // bit      description
    uint16_t MUX_SEL:8;             // 7~0
    uint16_t RSVH:8;                // 15~8
} DIAG_MUX_SEL_bits;

typedef union  {
   uint16_t                 all;
   DIAG_MUX_SEL_bits        bit;
} DIAG_MUX_SEL_t;

typedef struct
{                                   // bit      description
    uint16_t TOKEN_SEED:4;          // 3~0
    uint16_t FDBK:4;                // 7~4
    uint16_t RSVH:8;                // 15~8
} WD_TOKEN_FDBK_bits;

typedef union  {
   uint16_t                 all;
   WD_TOKEN_FDBK_bits       bit;
} WD_TOKEN_FDBK_t;

typedef struct
{                                   // bit      description
    uint16_t RT:7;                  // 6~0
    uint16_t RSV1:1;                // 7
    uint16_t RSVH:8;                // 15~8
} WD_WIN1_CFG_bits;

typedef union  {
   uint16_t                 all;
   WD_WIN1_CFG_bits         bit;
} WD_WIN1_CFG_t;

typedef struct
{                                   // bit      description
    uint16_t RW:5;                  // 4~0
    uint16_t RSV1:3;                // 7~5
    uint16_t RSVH:8;                // 15~8
} WD_WIN2_CFG_bits;

typedef union  {
   uint16_t                 all;
   WD_WIN2_CFG_bits         bit;
} WD_WIN2_CFG_t;

typedef struct
{                                   // bit      description
    uint16_t TOKEN:4;               // 3~0
    uint16_t RSV1:3;                // 6~4
    uint16_t WD_FAIL_TH:1;          // 7
    uint16_t RSVH:8;                // 15~8
} WD_TOKEN_VALUE_bits;

typedef union  {
   uint16_t                 all;
   WD_TOKEN_VALUE_bits      bit;
} WD_TOKEN_VALUE_t;

typedef struct
{                                   // bit      description
    uint16_t ANSWER_EARLY:1;        // 0
    uint16_t TIME_OUT:1;            // 1
    uint16_t SEQ_ERR:1;             // 2
    uint16_t WD_CFG_CHG:1;          // 3
    uint16_t WD_WRONG_CFG:1;        // 4
    uint16_t ANSWER_ERR:1;          // 5
    uint16_t WD_ANSW_CNT:2;         // 7~6
    uint16_t RSVH:8;                // 15~8
} WD_STATUS_bits;

typedef union  {
   uint16_t                 all;
   WD_STATUS_bits           bit;
} WD_STATUS_t;

typedef struct
{                                   // bit      description
    uint16_t WD_ANSW:8;             // 7~0
    uint16_t RSVH:8;                // 15~8
} WD_ANSWER_bits;

typedef union  {
   uint16_t                 all;
   WD_ANSWER_bits           bit;
} WD_ANSWER_t;

typedef struct
{                                   // bit      description
    uint16_t VSOUT1_EN:1;           // 0
    uint16_t RSV1:3;                // 3~1
    uint16_t VDD5_EN:1;             // 4
    uint16_t RSV2:3;                // 7~5
    uint16_t RSVH:8;                // 15~8
} SENS_CTRL_bits;

typedef union  {
   uint16_t                 all;
   SENS_CTRL_bits           bit;
} SENS_CTRL_t;

typedef struct  _TPS65381A_Vars_t_
{
    uint16_t                DEV_REV;            //!<
    uint16_t                DEV_ID;             //!<
    DEV_STAT_t              DEV_STAT;           //!<
    DEV_CFG1_t              DEV_CFG1;           //!<
    DEV_CFG2_t              DEV_CFG2;           //!<
    VMON_STAT_1_t           VMON_STAT_1;        //!<
    VMON_STAT_2_t           VMON_STAT_2;        //!<
    SAFETY_STAT_1_t         SAFETY_STAT_1;      //!<
    SAFETY_STAT_2_t         SAFETY_STAT_2;      //!<
    SAFETY_STAT_3_t         SAFETY_STAT_3;      //!<
    SAFETY_STAT_4_t         SAFETY_STAT_4;      //!<
    SAFETY_STAT_5_t         SAFETY_STAT_5;      //!<
    SAFETY_ERR_CFG_t        SAFETY_ERR_CFG;     //!<
    SAFETY_BIST_CTRL_t      SAFETY_BIST_CTRL;   //!<
    SAFETY_CHECK_CTRL_t     SAFETY_CHECK_CTRL;  //!<
    SAFETY_FUNC_CFG_t       SAFETY_FUNC_CFG;    //!<
    SAFETY_ERR_STAT_t       SAFETY_ERR_STAT;    //!<
    SAFETY_ERR_PWM_H_t      SAFETY_ERR_PWM_H;   //!<
    SAFETY_ERR_PWM_L_t      SAFETY_ERR_PWM_L;   //!<
    SAFETY_PWD_THR_CFG_t    SAFETY_PWD_THR_CFG; //!<
    SAFETY_CFG_CRC_t        SAFETY_CFG_CRC;     //!<
    DIAG_CFG_CTRL_t         DIAG_CFG_CTRL;      //!<
    DIAG_MUX_SEL_t          DIAG_MUX_SEL;       //!<
    WD_TOKEN_FDBK_t         WD_TOKEN_FDBK;      //!<
    WD_WIN1_CFG_t           WD_WIN1_CFG;        //!<
    WD_WIN2_CFG_t           WD_WIN2_CFG;        //!<
    WD_TOKEN_VALUE_t        WD_TOKEN_VALUE;     //!<
    WD_STATUS_t             WD_STATUS;          //!<
    WD_ANSWER_t             WD_ANSWER;          //!<
    SENS_CTRL_t             SENS_CTRL;          //!<

    WDState_e               wdTrigState;        //!<
    uint16_t                wdTrigTimeCnt;      //!<
    uint16_t                wdTrigTimeSet;      //!<

    bool                    readCtrlCmd;        //!<
    bool                    readStatCmd;        //!<
    bool                    writeCtrlCmd;       //!<
    bool                    writeWdCfgCmd;          //!<
    bool                    ReadWdStatCmd;          //!<
} TPS65381A_Vars_t;

extern TPS65381A_Vars_t gTps65381aVars;

void TPS65381A_InitRegister(TPS65381A_Vars_t *tps65381a);
void TPS65381A_Init(TPS65381A_Vars_t *tps65381a);
void TPS65381A_ReadSafetyStatus(TPS65381A_Vars_t *tps65381a);
void TPS65381A_ReadWdStatus(TPS65381A_Vars_t *tps65381a);

#endif /* TPS65381_H_ */
