/*
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * @file   tps65381_spi.c
 * @author Christian Herget
 * @date   2/3/2017
 * @brief  TPS65381 driver, provides functions to access registers via SPI
 *
 */
#include "F2806x_Device.h"     // F2806x Headerfile Include File

#include "sw/modules/types/src/types.h"
#include "sw/modules/usDelay/src/32b/usDelay.h"

#include "sw/safety/tps65381a/src/32b/f28x/f2806x/tps65381a_spi.h"

// ****************************************************************************
// SPI Configuration
// ****************************************************************************
void TPS65381A_InitSPI(void)
{
    SpiaRegs.SPICCR.bit.SPISWRESET = 0;       // Put SPI in reset state
    SpiaRegs.SPICCR.bit.SPICHAR = 0xF;        // 16-bit character
    SpiaRegs.SPICCR.bit.SPILBK = 0;           // loopback off
    SpiaRegs.SPICCR.bit.CLKPOLARITY = 0;      // Rising edge without delay

    SpiaRegs.SPICTL.bit.SPIINTENA = 0;        // disable SPI interrupt
    SpiaRegs.SPICTL.bit.TALK = 1;             // enable transmission
    SpiaRegs.SPICTL.bit.MASTER_SLAVE = 1;     // master
    SpiaRegs.SPICTL.bit.CLK_PHASE = 0;        // Rising edge without delay
    SpiaRegs.SPICTL.bit.OVERRUNINTENA = 0;    // disable reciever overrun interrupt

    SpiaRegs.SPIBRR = 44;                     // SPICLK = LSPCLK / 4 (max SPICLK), 90M/(44+1)=2MSPS
    SpiaRegs.SPICCR.bit.SPISWRESET=1;         // Enable SPI
}

/*****************************************************************************/
// SPI driver to interface with DRV830x
/*****************************************************************************/
uint16_t TPS65381A_SPI_Driver(uint16_t data)
{
    uint16_t dataread;
    uint16_t n;

    GpioDataRegs.GPBCLEAR.bit.GPIO39 = 1;;      // Enable SCS

    for(n=0; n<0x10; n++)
        asm(" NOP");

    // empty RX and TX FIFOs
    SpiaRegs.SPIFFTX.bit.TXFIFO = 0; // reset FIFOs
    SpiaRegs.SPIFFRX.bit.RXFIFORESET = 0;
    asm(" NOP");
    asm(" NOP");
    SpiaRegs.SPIFFTX.bit.TXFIFO = 1;
    SpiaRegs.SPIFFRX.bit.RXFIFORESET = 1; // release from reset

    SpiaRegs.SPITXBUF = data;                   // send out the data

    while (SpiaRegs.SPIFFRX.bit.RXFFST < 1)     // wait for the packet to complete
    {
    }

    dataread = SpiaRegs.SPIRXBUF;               // data read to clear the INT_FLAG bit

    for(n=0; n<0x1F; n++)
        asm(" NOP");

    GpioDataRegs.GPBSET.bit.GPIO39 = 1;         // Disable SCS

    return(dataread);
}

/*****************************************************************************/
// Read from DRV8305 Register at a certain address
/*****************************************************************************/
uint16_t TPS65381A_SPI_Read(uint16_t address)
{
    union TPS65381A_SPI_WRITE_WORD_REG w;

    w.bit.ADDRESS = address;                // load the address
    w.bit.DATA = 0;                         // data to be written

    return(TPS65381A_SPI_Driver(w.all));
}

/*****************************************************************************/
// Write to a DRV8305 Register at a certain address
/*****************************************************************************/
uint16_t TPS65381A_SPI_Write(uint16_t address, uint16_t data)
{
    union TPS65381A_SPI_WRITE_WORD_REG w;

    w.bit.ADDRESS = address;                // load the address
    w.bit.DATA = data;                      // data to be written

    return(TPS65381A_SPI_Driver(w.all));
}


