/*
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * @file   tpsQaWd.c
 * @author Christian Herget
 * @date   2/3/2017
 * @brief  TPS65381 Question and Answer Watchdog driver
 *
 */

#include <stdint.h>
#include <tpsQaWd.h>

#include "tps65381.h"
#include "functions.h"

#include "sys_common.h"
#include "mibspi.h"
#include "rti.h"
#include "esm.h"
#include "sci.h"
#include "gio.h"

volatile TPSState_t g_sTpsState    = TPS_STANDBY_STATE;
volatile uint8_t g_ui8FailCnt      = 0xFFu;
volatile uint8_t g_ui8BistTriggerd = 0u;
volatile uint8_t g_ui8DevErrorStat = 0u;
volatile uint8_t g_ui8SafetyStats3 = 0xFFu;

uint32_t windwCnt = 0x0U, g_goToActiveState = 0u;
static WDWindow_t WDWindow = open;

uint8_t tpsQaWdGetAnswrCnt(void)
{
    return ((tpsReadRegister(RD_WDT_STATUS) & 0xC0) >> 6);
}

/* read WD Token and return it */
uint8_t tpsQaWdGetToken(void)
{
    uint8_t ui8Token = 0u;

    ui8Token = tpsReadRegister(RD_WDT_TOKEN_VALUE);

    return (ui8Token & 0x0Fu);
}

/** @fn uint8_t tpsQaWdCalcAnswr(uint8_t ui8Token)
 *  @brief calculates WD answer
 *
 *  @param[in] ui8Token - read out token
 *
 *  @return calculated answer
 *
 *  This function calculates the expected WD
 *  answer from a given token
 */
uint8_t tpsQaWdCalcAnswr(uint8_t ui8Token)
{
    uint8_t ui8Cnt = 0;
    uint8_t ui8Response = 0;
    uint8_t ui8Answr[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };

    ui8Cnt = tpsQaWdGetAnswrCnt();

    /* Create WD answer */
    ui8Answr[0] = ((ui8Token & 0x1))
            ^ (((ui8Cnt & 0x2) >> 1) ^ ((ui8Token & 0x8) >> 3));
    ui8Answr[1] = ((ui8Token & 0x1) << 1)
            ^ (((ui8Token & 0x4) >> 1) ^ (ui8Token & 0x2)) ^ ((ui8Cnt & 0x2));
    ui8Answr[2] = ((ui8Token & 0x1) << 2)
            ^ (((ui8Token & 0x8) >> 1) ^ ((ui8Token & 0x2) << 1))
            ^ ((ui8Cnt & 0x2) << 1);
    ui8Answr[3] = ((ui8Token & 0x4) << 1)
            ^ (((ui8Token & 0x1) << 3) ^ (ui8Token & 0x8))
            ^ ((ui8Cnt & 0x2) << 2);
    ui8Answr[4] = ((ui8Token & 0x2) << 3) ^ ((ui8Cnt & 0x1) << 4);
    ui8Answr[5] = ((ui8Token & 0x8) << 2) ^ ((ui8Cnt & 0x1) << 5);
    ui8Answr[6] = ((ui8Token & 0x1) << 6) ^ ((ui8Cnt & 0x1) << 6);
    ui8Answr[7] = ((ui8Token & 0x4) << 5) ^ ((ui8Cnt & 0x1) << 7);

    ui8Response = ui8Answr[0] | ui8Answr[1] | ui8Answr[2] | ui8Answr[3]
            | ui8Answr[4] | ui8Answr[5] | ui8Answr[6] | ui8Answr[7];

    return ui8Response;
}

/** @fn uint8_t tpsQaWdConfigure(uint8_t ui8Topen, uint8_t ui8Tclose)
 *  @brief Set WD window registers
 *
 *  @param[in]  ui8Topen  - Open Window Time
 *  @param[in]  ui8Tclose - Closed Window Time
 *
 *  @return True if the windows where successfully set
 *
 *  sets WD window register and evaluates set values,
 *  returns 1 if values are set correctly to registers
 */

uint8_t tpsQaWdConfigure(uint8_t ui8Topen, uint8_t ui8Tclose)
{
    uint8_t ui8Check;

    tpsWriteRegister(WR_OPEN_WD, ui8Topen);
    tpsWriteRegister(WR_CLOSE_WD, ui8Tclose);

    ui8Check = tpsIsRegisterSetToVal(RD_WD_OPEN_WIN_CFG, ui8Topen)
            && tpsIsRegisterSetToVal(RD_WD_CLOSE_WIN_CFG, ui8Tclose);

    return ui8Check;
}

void setDelay(uint32_t delay)
{
    /*time in us */
    delay = delay * 10;
    rtiREG1->CMP[2U].COMPx = delay;
    rtiREG1->CMP[2U].UDCPx = delay;
}

/* send calculated answer to TPS chip */
void tpsQaWdOpenWinAnswr(void)
{
    /* set WDAnswr */
    tpsWriteRegister(WR_WDT_ANSWER, tpsQaWdCalcAnswr(tpsQaWdGetToken()));
}

/* send first three answers to TPS while WD window is open */
void openwindow(void)
{
    /*wait for open window */
    while (tpsQaWdIsTimeOut())
        ;

    tpsQaWdOpenWinAnswr();
    tpsQaWdOpenWinAnswr();
    tpsQaWdOpenWinAnswr();
}

/* send last answer  to TPS while WD window is closed      */
void tpsQaWdCloseWinAnswr(void)
{
    tpsWriteRegister(WR_WDT_ANSWER, tpsQaWdCalcAnswr(tpsQaWdGetToken()));
}

/** @fn void tpsQaWdLaunchDemo(void)
 *  @brief Demo application implements state machine to bring TPS into Active State and to perform some self-test
 *
 *  This function contains a state machine to control the TPS and all SCI Terminal print function which are executed asynchronously to the TPS accesses
 */
void tpsQaWdLaunchDemo(void)
{
    tpsUnlockWriteAccess();

    g_sTpsState = (TPSState_t) (tpsReadRegister(RD_SAFETY_STAT_5) & 0x0007u);

    if (TPS_DIAGNOSTIC_STATE == g_sTpsState)
    {
        /* open win = 7.8ms, closed win 2.6ms */
        while (!tpsQaWdConfigure(0x4E, 0x1A));

        rtiEnableNotification(rtiNOTIFICATION_COMPARE2);
        setDelay(1000);/* -> 1.0ms */

        rtiStartCounter(rtiCOUNTER_BLOCK1);

        /* Ensure, that ENDRV is disabled */
        tpsClearEnDrv();

        /* enable WD Q&A mode */
        tpsWriteRegister(WR_SAFETY_FUNC_CFG, 0xA0); //A0 2 disable WD (0xA8 2 enable)

        //esmREG->EKR = 0x5U;
        /* clear error status registers */
        tpsQaWdClearErrorCnt();

        /* wait for interrupt */
        while (1)
        {
            TPSState_t sCurTpsState       = g_sTpsState;
            uint8_t    ui8CurFailCnt      = g_ui8FailCnt;
            uint8_t    ui8CurDevErrorStat = g_ui8DevErrorStat;
            uint8_t    ui8CurSpiStat      = g_ui8SpiStat;

            static TPSState_t sLastTpsState    = TPS_STANDBY_STATE;
            static uint8_t ui8LastFailCnt      = 0xFFu;
            static uint8_t ui8LastDevErrorStat = 0u;

            static uint8_t u8EnDrvTest = 0u;

            if (sLastTpsState != sCurTpsState)
            {
                sciPrintDevState(sCurTpsState);
                sLastTpsState = sCurTpsState;
            }

            if (ui8LastFailCnt != ui8CurFailCnt)
            {
                sciPrintf("FailCnt: %d\r\n", (ui8CurFailCnt));
                ui8LastFailCnt = ui8CurFailCnt;
            }

            if (0u != ui8CurDevErrorStat)
            {
                sciPrintf("Clear SAFETY_ERR_STAT 0x%02X\r\n", ui8CurDevErrorStat);

                /* TODO: Move access to TPS to rtiNotification */
                tpsQaWdClearErrorCnt();

                ui8LastDevErrorStat = ui8CurDevErrorStat;
            }
            else if (ui8LastDevErrorStat != ui8CurDevErrorStat)
            {
                sciPrintf("SAFETY_ERR_STAT 0x%02X\r\n", ui8CurDevErrorStat);

                ui8LastDevErrorStat = ui8CurDevErrorStat;
            }

            if (0xA0u != (ui8CurSpiStat & 0xF7u))
            {
                sciPrintf("SPI STAT 0x%02X\r\n", ui8CurSpiStat);
            }

            if ((0u == g_ui8FailCnt) && (TPS_DIAGNOSTIC_STATE == g_sTpsState))
            {
                if (0u == u8EnDrvTest)
                {
                    sciPrint("Starting ENDRV Test\r\n");
                    u8EnDrvTest++;
                }
                else if (1u == u8EnDrvTest)
                {
                    sciPrint("Check for ENDRV = Low\r\n");

                    /* Read Back Low Level */
                    uint32_t u32EnDrvState = gioGetBit(gioPORTA, 4u);

                    if (0u == u32EnDrvState)
                    {
                        sciPrint("ENDRV = LOW --> PASS\r\n");

                        u8EnDrvTest++;
                    }
                    else
                    {
                        sciPrint("ENDRV = HIGH --> FAIL\r\n");
                    }
                }
                else if (2u == u8EnDrvTest)
                {
                    tpsSetEnDrv();

                    u8EnDrvTest++;
                }
                else if (3u == u8EnDrvTest)
                {
                    sciPrint("Check for ENDRV = High\r\n");

                    /* Read Back Low Level */
                    uint32_t u32EnDrvState = gioGetBit(gioPORTA, 4u);

                    if (0u == u32EnDrvState)
                    {
                        sciPrint("ENDRV = LOW --> FAIL\r\n");
                    }
                    else
                    {
                        sciPrint("ENDRV = HIGH --> PASS\r\n");

                        u8EnDrvTest++;
                    }
                }
                else if (4u == u8EnDrvTest)
                {
                    sciPrint("ENDRV Test PASSED\r\n");

                    u8EnDrvTest++;
                }
                else if (5u == u8EnDrvTest)
                {
                    sciPrint("Start BIST in DIAG State\r\n");

                    /* TODO: Move access to TPS to rtiNotification */
                    tpsWriteRegister(WR_SAFETY_BIST_CTRL, 0x07u);
                    rtiREG1->CMP[2U].COMPx += 22 * 1000 * 10;

                    g_ui8BistTriggerd = 1u;

                    u8EnDrvTest++;
                }
                else if ((6u == u8EnDrvTest) && (0u == g_ui8BistTriggerd))
                {
                    if (0u == g_ui8DevErrorStat)
                    {
                        sciPrint("BIST Finished and Passed\r\n");
                        u8EnDrvTest++;
                    }
                    else
                    {
                        sciPrintf("BIST Finished but Failed (0x%02X)\r\n",
                                  g_ui8DevErrorStat);
                    }
                }
                else if (7u == u8EnDrvTest)
                {
                    sciPrint("Switch to ACTIVE STATE\r\n");
                    g_goToActiveState = 1u;

                    u8EnDrvTest++;
                }
                else
                {
                    /* Do Nothing */
                }
            }
            else if ((0u == g_ui8FailCnt) && (TPS_ACTIVE_STATE == g_sTpsState))
            {
                if (8u == u8EnDrvTest)
                {
                    sciPrint("Start BIST in ACTIVE State\r\n");

                    /* TODO: Move access to TPS to rtiNotification */
                    tpsWriteRegister(WR_SAFETY_BIST_CTRL, 0x07u);
                    rtiREG1->CMP[2U].COMPx += 22 * 1000 * 10;

                    g_ui8BistTriggerd = 1u;

                    u8EnDrvTest++;
                }
                else if ((9u == u8EnDrvTest) && (0u == g_ui8BistTriggerd))
                {
                    if (0u == g_ui8DevErrorStat)
                    {
                        sciPrint("BIST Finished and Passed\r\n");
                        u8EnDrvTest++;
                    }
                    else
                    {
                        sciPrintf("BIST Finished but Failed (0x%02X)\r\n",
                                  g_ui8DevErrorStat);
                    }
                }
                else if (10u == u8EnDrvTest)
                {

                }
                else
                {
                    /* Do Nothing */
                }
            }
            else
            {
                /* Do Nothing */
            }
        }
    }
    else
    {
        sciPrint("TPS65381 is not in DIAGNOSTIC state - toggle IGN!\r\n");
    }
}

/** @fn void rtiNotification(uint32_t notification)
 *  @brief RIT Interrupt Call Out
 *
 *  This function contains all functions which are accessing the TPS via SPI while in "real time mode" to avoid arbitration issues
 */
void rtiNotification(uint32_t notification)
{
    if (rtiNOTIFICATION_COMPARE2 == notification)
    {
        if (1u == g_ui8BistTriggerd)
        {
            g_ui8SafetyStats3 = tpsReadRegister(RD_SAFETY_STAT_3);

            if (0u != (g_ui8SafetyStats3 & 0x03u))
            {
                /* At least one BIST is still running */

                rtiREG1->CMP[2U].UDCPx = 10000u; /* wait 1 ms */
            }
            else
            {
                if (TPS_DIAGNOSTIC_STATE == g_sTpsState)
                {
                    /* Stay in Diag State and restore config */
                    tpsWriteRegister(WR_SAFETY_CHECK_CTRL, 0x36u);
                    //tpsStayInDiagState();
                }
                else if (TPS_ACTIVE_STATE == g_sTpsState)
                {
                    /* Restore config */
                    tpsWriteRegister(WR_SAFETY_CHECK_CTRL, 0x34u);
                }
                else
                {
                    /* BIST executed in wrong mode? */
                }

                /* Cause a bad event to re-synchronize and start a new watchdog sequence */
                tpsWriteRegister(WR_WDT_ANSWER, 0x00u);
                WDWindow = open;

                /* Done */
                g_ui8BistTriggerd = 0u;
            }
        }
        else
        {
            g_sTpsState = (TPSState_t) (tpsReadRegister(RD_SAFETY_STAT_5) & 0x07u);
            g_ui8FailCnt = tpsQaWdReadFailCount();
            g_ui8DevErrorStat = tpsReadRegister(RD_SAFETY_ERR_STAT) & 0x3Fu;
        }

        switch (WDWindow)
        {
        case open:

            openwindow();
            rtiREG1->CMP[2U].UDCPx = 500000u; /* wait 50 ms */

            WDWindow = close;
            break;

        case close:
        {
            if ((0u == g_ui8FailCnt) && (TPS_DIAGNOSTIC_STATE == g_sTpsState)
                    && (1u == g_goToActiveState))
            {
                /* Important for some reason */
                tpsQaWdClearErrorCnt();

                /* enable Q&A WD */
                tpsWriteRegister(WR_SAFETY_FUNC_CFG, 0xA8); //0xA0 2 disable WD (0xA8 2 enable);

                tpsGotoActiveState();
            }

            tpsQaWdCloseWinAnswr();

            rtiREG1->CMP[2U].UDCPx = 30000u; /* wait 3 ms */

            WDWindow = open;
        }

            break;
        }
    }
}

/** @fn uint16_t tpsQaWdIsTimeOut(void)
 *   @brief reads out TO flag
 *
 *   @return TO flag
 *
 *   This function reads out TPS65381 time
 *   out flag and returns 1 when Timeout Flag
 *   in WDT_STATUS register is set
 */
uint16_t tpsQaWdIsTimeOut(void)
{
    return ((tpsReadRegister(RD_WDT_STATUS) & 0x02) >> 1);
}

uint8_t tpsQaWdReadFailCount(void)
{
    uint8_t FailCount;
    FailCount = tpsReadRegister(RD_SAFETY_STAT_2);

    return (FailCount & 0x7);
}

void tpsQaWdClearErrorCnt(void)
{
    tpsWriteRegister(WR_SAFETY_ERR_STAT, 0x00);
}
